@@ -721,16 +721,25 @@ void btn_mode_pressed(void)
 		}
 }
 
-void btn_edit_skin_pressed(void)
+void btn_edit_skin_pressed(UIData *ui)
 {
 	EditData *ed;
+	GList *list = NULL;
 
-	ed = ui_edit_new(main_window, "GQmpeg", "skin_editor", _("Skin editor - GQmpeg"));
+	ed = ui_edit_new(ui, "GQmpeg", "skin_editor", _("Skin editor - GQmpeg"));
 
 	/* ed is NULL if an edit window is already active for the ui */
 	if (!ed) return;
+
 	ui_edit_set_help_path(ed, GQMPEG_SKIN_HELP_PATH);
+
+	list = g_list_append(list, "skindata");
+	list = g_list_append(list, "skindata_menu");
+	list = g_list_append(list, "skindata_flyby");
+	list = g_list_append(list, "skindata_alt");
+
+	ui_edit_set_skindata_list(ed, list);
+
+	g_list_free(list);
 }
 
-
-
