@@ -75,7 +75,7 @@ void set_preset_to_playlist(void);
 void set_preset_to_song(void);
 
 void btn_mode_pressed(void);
-void btn_edit_skin_pressed(void);
+void btn_edit_skin_pressed(UIData *ui);
 
 
 #endif
