@@ -81,13 +81,13 @@ gfloat check_child_cpu_usage(int childs_pid)
 	    if(!strcmp(mntbufp[fs_count].f_fstypename, "procfs") &&
 #endif
 	       (procfs_found = 1))
-	      sprintf(procfs_mountpoint, mntbufp[fs_count].f_mntonname);
+	      snprintf(procfs_mountpoint, sizeof(procfs_mountpoint), mntbufp[fs_count].f_mntonname);
 	  
 	  if(!procfs_found)
 	    return 0.0;
 	}
 
-	sprintf(procname, "%s/%d/status", procfs_mountpoint, childs_pid); /* no overflow */
+	snprintf(procname, sizeof(procname), "%s/%d/status", procfs_mountpoint, childs_pid); /* no overflow */
 	
 	if ( (procfile = open(procname, O_RDONLY)) == -1 )
 		;
@@ -173,7 +173,7 @@ gfloat check_child_cpu_usage(int childs_pid)
 	char procname[20];
 	int  procfile;
   
-	sprintf(procname, "/proc/%.5d", childs_pid);
+	snprintf(procname, sizeof(procname), "/proc/%.5d", childs_pid);
 
 	if ( (procfile = open(procname, O_RDONLY)) == -1 )
 		percent_of_cpu = 0.0;
@@ -206,12 +206,12 @@ gfloat check_child_cpu_usage(int childs_pid)
 
 	gettimeofday(&time, &timez);
 
-	sprintf(s,"/proc/%d/stat",childs_pid);
+	snprintf(s, sizeof(s), "/proc/%d/stat", childs_pid);
 
 /* needed for SGI IRIX 5.3 and 6.5 */
 #ifdef SGI
 
-	sprintf(s,"/proc/pinfo/%05d",childs_pid);
+	snprintf(s, sizeof(s), "/proc/pinfo/%05d",childs_pid);
 
 	fd = open(s, O_RDONLY);
 	if (fd < 0) return 0;
