@@ -492,6 +492,11 @@ static const gchar *text_comment_status_cb(TextData *text, const gchar *key, gpo
 	return current_song_get_comment();
 }
 
+static const gchar *text_playlist_status_cb(TextData *text, const gchar *key, gpointer data)
+{
+	return playlist_filename;
+}
+
 /* sliders */
 
 static gfloat slider_position_status_cb(SliderData *slider, const gchar *key, gpointer data)
@@ -571,11 +576,11 @@ static gint list_playlist_length_request_cb(ListData *list, const gchar *key, gp
 	return playlist_get_count();
 }
 
-static gint flag_convert(gint row)
+static gint flag_convert(SongData *sd)
 {
 	SongFlags flags;
 
-	flags = playlist_get_flags(row);
+	flags = sd->flags;
 
 	if (flags & SONG_FLAG_PLAYING) return 0;
 	if (flags & SONG_FLAG_NOT_FOUND) return 3;
@@ -588,17 +593,23 @@ static gint flag_convert(gint row)
 static gint list_playlist_row_info_cb(ListData *list, const gchar *key,
 				      gint row, ListRowData *rd, gpointer data)
 {
+	SongData *sd;
 	gchar *buf;
 
-	list_row_column_set_text(list, rd, "title", playlist_get_title(row));
-	buf = playlist_get_time(row);
+	sd = playlist_get_data(row);
+	if (!sd) return FALSE;
+
+	list_row_column_set_text(list, rd, "title", playlist_data_get_title(sd));
+	buf = playlist_data_get_time(sd);
 	list_row_column_set_text(list, rd, "time", buf);
 	g_free(buf);
 	buf = g_strdup_printf("%d.", row + 1);
 	list_row_column_set_text(list, rd, "number", buf);
 	g_free(buf);
 
-	list_row_set_flag(rd, flag_convert(row));
+	list_row_set_flag(rd, flag_convert(sd));
+
+	playlist_update_background(sd);
 
 	return TRUE;
 }
@@ -844,6 +855,8 @@ void display_register_widgets(UIData *ui)
 	text_register_key("year", ui, text_year_status_cb, NULL);
 	text_register_key("comment", ui, text_comment_status_cb, NULL);
 
+	text_register_key("playlist", ui, text_playlist_status_cb, NULL);
+
 	/* sliders */
 
 	slider_register_key("position", ui,
@@ -1046,18 +1059,32 @@ void display_set_total_mode(void)
 
 void display_set_position(gfloat value)
 {
+	gint total;
+
+	total = seconds_remaining + seconds;
+
 	if (value >= 0.0)
+		{
 		dsp_position = value;
-	else if (seconds_remaining == 0 && seconds == 0)
+		}
+	else if (total == 0)
+		{
 		dsp_position = 0.0;
+		}
 	else
-		dsp_position = (float)seconds / (seconds_remaining + seconds);
+		{
+		dsp_position = (float)seconds / total;
+		}
 
 	if (!main_window) return;
 
 	item_section_set_by_percent("position", main_window, dsp_position);
 	dial_value_set("position", main_window, dsp_position);
 	slider_value_set("position", main_window, dsp_position);
+	if (total > 0)
+		{
+		slider_step_size_set("position", main_window, (1.0 / (gfloat)total), 10);
+		}
 }
 
 void display_set_volume(void)
@@ -1212,12 +1239,18 @@ void display_set_io_information(gint zero)
 
 	if (zero)
 		{
-		input_bitrate = 0;
-		input_hz = 0;
+		SongData *sd;
+
+		sd = current_song_get_data();
+
+		input_bitrate = (sd) ? sd->bit_rate : 0;
+		input_hz = (sd) ? sd->khz_rate : 0;
 #if 0
 		frames = frames_remaining = 0;
-		seconds = seconds_remaining = 0;
 #endif
+		seconds =  0;
+		seconds_remaining = (sd) ? sd->length : 0;
+
 		output_channels = 0;
 		mpeg_version = 0;
 		mpeg_layer = 0;
@@ -1277,6 +1310,13 @@ void display_set_title(const gchar *text)
 	text_set_autoscroll("title", main_window, title_scrolls_always);
 }
 
+void display_set_playlist(const gchar *text)
+{
+	if (!main_window) return;
+
+	text_set_text("playlist", main_window, text);
+}
+
 /*
  *-----------------------------------------------------------------------------
  * display utility functions (public)
