@@ -256,7 +256,7 @@ void dnd_init(void)
 			main_drop_types, main_drop_count, 
 			GDK_ACTION_COPY);
 	gtk_signal_connect(GTK_OBJECT(main_window->window), "drag_data_received",
-			dnd_main_get_data , NULL);
+			   GTK_SIGNAL_FUNC(dnd_main_get_data), NULL);
 }
 
 
