@@ -230,7 +230,7 @@ const gchar *ogg123_comment_value(GList *list, const gchar *key)
 		{
 		const gchar *poss = work->data;
 
-		if (strcmp(poss, key) == 0)
+		if (strcasecmp(poss, key) == 0)
 			{
 			if (work->next)
 				{
@@ -434,6 +434,7 @@ static void ogg123_child_shutdown(void)
 	if (ogg123_control_read_id != -1) gdk_input_remove(ogg123_control_read_id);
 	ogg123_control_read_id = -1;
 	close(ogg123_pipe[0]);
+	ogg123_pipe[0] = -1;
 
 	ogg123_child_pid = -1;
 	pid = 0;
@@ -651,39 +652,42 @@ static gint ogg123_child_run(SongData *sd, gint position)
 	char cmd_arguments[OGG123_MAX_COMMANDS][512];
 	char *cmd_ptr[OGG123_MAX_COMMANDS];
 	int cmd_cnt = 0;
+	size_t cmd_max;
 	gchar *exec_bin = OGG123_BINARY;
 
+	cmd_max = sizeof(cmd_arguments) / OGG123_MAX_COMMANDS;
+
 	if (!sd || ogg123_child_pid != -1) return FALSE;
 
-	strcpy(cmd_arguments[cmd_cnt], OGG123_BINARY);
+	strncpy(cmd_arguments[cmd_cnt], OGG123_BINARY, cmd_max);
 	cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 	cmd_cnt++;
 
-	strcpy(cmd_arguments[cmd_cnt], "-v");
+	strncpy(cmd_arguments[cmd_cnt], "-v", cmd_max);
 	cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 	cmd_cnt++;
 
 	if (position > 0)
 		{
-		strcpy(cmd_arguments[cmd_cnt], "-k");
+		strncpy(cmd_arguments[cmd_cnt], "-k", cmd_max);
 		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 		cmd_cnt++;
 
-		sprintf(cmd_arguments[cmd_cnt], "%d", position);
+		snprintf(cmd_arguments[cmd_cnt], cmd_max, "%d", position);
 		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 		cmd_cnt++;
 		}
 
 	if (strcmp(ogg123_get_device(ogg123_device), "default") != 0)
 		{
-		sprintf(cmd_arguments[cmd_cnt], "--device=%s", ogg123_get_device(ogg123_device));
+		snprintf(cmd_arguments[cmd_cnt], cmd_max, "--device=%s", ogg123_get_device(ogg123_device));
 		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 		cmd_cnt++;
 		}
 
 	if (ogg123_device_options && strlen(ogg123_device_options) > 0)
 		{
-		sprintf(cmd_arguments[cmd_cnt], "--device-option=%s", ogg123_device_options);
+		snprintf(cmd_arguments[cmd_cnt], cmd_max, "--device-option=%s", ogg123_device_options);
 		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 		cmd_cnt++;
 		}
@@ -700,7 +704,7 @@ static gint ogg123_child_run(SongData *sd, gint position)
 
 		while(vector && vector[i] != NULL)
 			{
-			strcpy(cmd_arguments[cmd_cnt], vector[i]);
+			strncpy(cmd_arguments[cmd_cnt], vector[i], cmd_max);
 			cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 			cmd_cnt++;
 			i++;
@@ -708,11 +712,11 @@ static gint ogg123_child_run(SongData *sd, gint position)
 		g_strfreev(vector);
 		}
 
-	strcpy(cmd_arguments[cmd_cnt], sd->path);
+	strncpy(cmd_arguments[cmd_cnt], sd->path, cmd_max);
 	cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 	cmd_cnt++;
 
-	strcpy(cmd_arguments[cmd_cnt], "");
+	strncpy(cmd_arguments[cmd_cnt], "", cmd_max);
 	cmd_ptr[cmd_cnt] = NULL;
 	cmd_cnt++;
 
@@ -754,6 +758,7 @@ static gint ogg123_child_run(SongData *sd, gint position)
 		pid = ogg123_child_pid;
 
 		close(ogg123_pipe[1]);
+		ogg123_pipe[1] = -1;
 		}
 
 	if (debug_mode) printf("ogg123 pid = %d\n", ogg123_child_pid);
