@@ -1,6 +1,6 @@
 /*
  * GQmpeg
- * (C) 2001 John Ellis
+ * (C) 2002 John Ellis
  *
  * Author: John Ellis
  *
@@ -14,6 +14,20 @@
 #define IO_OGG123_H
 
 
+typedef struct _OGGInfo OGGInfo;
+struct _OGGInfo
+{
+	gint pass_integrity;
+	gint channels;
+	gint bitrate_lower;
+	gint bitrate_upper;
+	gint bitrate_average;
+	gint bitrate_nominal;
+	gint length;		/* seconds */
+	gint bytes;		/* file size */
+};
+
+
 extern gint ogg123_enabled;
 
 extern gint ogg123_device;
@@ -24,10 +38,14 @@ extern gchar *ogg123_extra_options;
 /*
  * =========================== io_ogg123.c
  */
-
 const gchar *ogg123_get_device(gint device);
 GList *ogg123_get_device_list(void);
 
+OGGInfo *ogg123_info_get(const gchar *path);
+GList *ogg123_comment_get(const gchar *path);
+const gchar *ogg123_comment_value(GList *list, const gchar *key);
+gint ogg123_comment_set(const gchar *path, GList *comments);
+
 void ogg123_init(void);
 
 /*
@@ -39,6 +57,12 @@ void ogg123_config_apply(void);
 void ogg123_config_close(void);
 GtkWidget *ogg123_config_init(void);
 
+/*
+ * =========================== ogg_info.c
+ */
+GtkWidget *ogg123_info_window(const gchar *file);
+
 
 #endif
 
+
