@@ -774,7 +774,8 @@ static GtkWidget *radio_entry_new(double freq, const gchar *title, gint scan)
         	gtk_widget_show(hbox);
 
 		button = gtk_button_new_with_label(_("Auto scan"));
-		gtk_signal_connect(GTK_OBJECT(button), "clicked", radio_entry_scan_cb, NULL);
+		gtk_signal_connect(GTK_OBJECT(button), "clicked",
+				   GTK_SIGNAL_FUNC(radio_entry_scan_cb), NULL);
 		gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 		gtk_widget_show(button);
 
