@@ -105,7 +105,8 @@ GtkWidget *radio_config_init(void)
 		}
 	else
 		{
-		gtk_signal_connect(GTK_OBJECT(button), "clicked", radio_config_redetect_cb, label);
+		gtk_signal_connect(GTK_OBJECT(button), "clicked",
+				   GTK_SIGNAL_FUNC(radio_config_redetect_cb), label);
 		}
 
 	if (!radio_enabled) gtk_widget_set_sensitive(radio_vbox, FALSE);
