@@ -440,88 +440,91 @@ static gint xmp_child_run(SongData *sd, gint position)
 	pid_t frk_pid;
 	char cmd_arguments[16][512];
 	char *cmd_ptr[16];
+	size_t cmd_max;
 	int cmd_cnt = 0;
         gint start_pattern;
 	gchar *exec_bin = XMP_BINARY;
 
+	cmd_max = sizeof(cmd_arguments) / 16;
+
 	if (!sd) return FALSE;
 
 	start_pattern = xmp_real_position(sd, position);
 
-	strcpy(cmd_arguments[cmd_cnt], XMP_BINARY);
+	strncpy(cmd_arguments[cmd_cnt], XMP_BINARY, cmd_max);
 	cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 	cmd_cnt++;
 
-	strcpy(cmd_arguments[cmd_cnt], "--norc");
+	strncpy(cmd_arguments[cmd_cnt], "--norc", cmd_max);
 	cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 	cmd_cnt++;
 
-	strcpy(cmd_arguments[cmd_cnt], "-v");
+	strncpy(cmd_arguments[cmd_cnt], "-v", cmd_max);
 	cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 	cmd_cnt++;
 
 	if (start_pattern > 0)
 		{
-		strcpy(cmd_arguments[cmd_cnt], "--start");
+		strncpy(cmd_arguments[cmd_cnt], "--start", cmd_max);
 		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 		cmd_cnt++;
 
-		sprintf(cmd_arguments[cmd_cnt], "%dX", start_pattern);
+		snprintf(cmd_arguments[cmd_cnt], cmd_max, "%dX", start_pattern);
 		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 		cmd_cnt++;
 		}
 
-	strcpy(cmd_arguments[cmd_cnt], "--frequency");
+	strncpy(cmd_arguments[cmd_cnt], "--frequency", cmd_max);
 	cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 	cmd_cnt++;
 
-	sprintf(cmd_arguments[cmd_cnt], "%d", xmp_frequency);
+	snprintf(cmd_arguments[cmd_cnt], cmd_max, "%d", xmp_frequency);
 	cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 	cmd_cnt++;
 
 	if (xmp_8bit)
 		{
-		strcpy(cmd_arguments[cmd_cnt], "--bits");
+		strncpy(cmd_arguments[cmd_cnt], "--bits", cmd_max);
 		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 		cmd_cnt++;
 
-		strcpy(cmd_arguments[cmd_cnt], "8");
+		strncpy(cmd_arguments[cmd_cnt], "8", cmd_max);
 		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 		cmd_cnt++;
 		}
 	if (xmp_interpolate)
 		{
-		strcpy(cmd_arguments[cmd_cnt], "--interpolate");
+		strncpy(cmd_arguments[cmd_cnt], "--interpolate", cmd_max);
 		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 		cmd_cnt++;
 		}
 
 	if (xmp_mono)
 		{
-		strcpy(cmd_arguments[cmd_cnt], "--mono");
+		strncpy(cmd_arguments[cmd_cnt], "--mono", cmd_max);
 		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 		cmd_cnt++;
 		}
 	else
 		{
-		strcpy(cmd_arguments[cmd_cnt], "--stereo");
+		strncpy(cmd_arguments[cmd_cnt], "--stereo", cmd_max);
 		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 		cmd_cnt++;
 		}
 
-	strcpy(cmd_arguments[cmd_cnt], "--pan");
+	strncpy(cmd_arguments[cmd_cnt], "--pan", cmd_max);
 	cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 	cmd_cnt++;
 
-	sprintf(cmd_arguments[cmd_cnt], "%d", xmp_pan_percent);
+	snprintf(cmd_arguments[cmd_cnt], cmd_max, "%d", xmp_pan_percent);
 	cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 	cmd_cnt++;
 
-	strcpy(cmd_arguments[cmd_cnt], sd->path);
+	strncpy(cmd_arguments[cmd_cnt], sd->path, cmd_max);
 	cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 	cmd_cnt++;
 
-	strcpy(cmd_arguments[cmd_cnt], "");
+	strncpy(cmd_arguments[cmd_cnt], "", cmd_max);
 	cmd_ptr[cmd_cnt] = NULL;
 	cmd_cnt++;
 
