@@ -15,6 +15,7 @@
 #include "btn_funcs.h"
 #include "cpu_perc.h"
 #include "display.h"
+#include "display_flyby.h"
 #include "dndutil.h"
 #include "dock.h"
 #include "filelist.h"
@@ -191,6 +192,7 @@ void current_song_set(gint n, gchar *path)
 	playlist_unset_flags(old_song, SONG_FLAG_PLAYING);
 	playlist_set_flags(current_song_get_number(), SONG_FLAG_PLAYING);
 	display_total_time_changed();
+	display_flyby_reset();
 }
 
 gint current_song_is_in_playlist(void)
@@ -413,6 +415,7 @@ static gint update_condition(gpointer data)
 					display_set_cpu(0);
 					display_set_frame_count(0);
 					display_set_io_information(TRUE);
+					display_flyby_reset();
 					}
 				else
 					{
@@ -432,6 +435,7 @@ static gint update_condition(gpointer data)
 				{
 				status = STATUS_STOP;
 				playback_exec_command(EXEC_PLAY, current_song_get_data(), 0);
+				display_flyby_start();
 				}
 			else
 				{
@@ -475,11 +479,13 @@ static gint update_condition(gpointer data)
 		display_set_io_information(TRUE);
 		display_set_position(-1.0);
 		display_set_status(FALSE, FALSE);
+		display_flyby_reset();
 		}
 	if (status == STATUS_PAUSE)
 		{
 		display_set_cpu(0);
 		display_set_status(FALSE, FALSE);
+		display_flyby_reset();
 		}
 	status_chk = status;
 	return TRUE;
@@ -491,19 +497,20 @@ static gint update_condition(gpointer data)
  *-----------------------------------------------------------------------------
  */
 
-static gchar *prepend_path_if_match(gchar *text)
+static gchar *prepend_path_if_match(const gchar *text)
 {
 	gchar *buf;
 
-	if (isfile(text))
-		{
-		return g_strdup(text);
-		}
-	buf = g_strconcat(current_path, "/", text, NULL);
-	if (isfile(buf))
+	if (!text) return NULL;
+
+	if (text[0] == '/') return g_strdup(text);
+
+	buf = concat_dir_and_file(current_path, text);
+	if (isname(buf))
 		{
 		return buf;
 		}
+
 	g_free(buf);
 	return g_strdup(text);
 }
@@ -917,7 +924,7 @@ void gqmpeg_exit(void)
 	states_save();
 
 	/* save playlist between sessions (even if option not enabled) */
-	session_file = g_strconcat(homedir(), "/.gqmpeg/session-playlist.gqmpeg", NULL);
+	session_file = g_strconcat(homedir(), "/", GQMPEG_RC_FILE_SESSION, NULL);
 	playlist_save(session_file);
 	g_free(session_file);
 
@@ -997,27 +1004,19 @@ int main (int argc, char *argv[])
 		i = 1;
 		while (i < argc)
 			{
-			gchar *command_line_path = g_strconcat(current_path, "/", argv[i], NULL);
+			gchar *command_line_path = prepend_path_if_match(argv[i]);
 
 			if (strncmp(argv[i], "-", 1) != 0)
 				{
-				if (isdir(command_line_path) || isdir(argv[i]))
+				if (isdir(command_line_path))
 					{
 					if (!default_current_path)
 						{
-						if (isdir(argv[i]))
-							default_current_path = g_strdup(argv[i]);
-						else
-							default_current_path = g_strdup(command_line_path);
+						default_current_path = g_strdup(argv[i]);
 						}
 					}
 				else
 					{
-					if (isfile(argv[i]) || !isfile(command_line_path))
-						{
-						g_free(command_line_path);
-						command_line_path = g_strdup(argv[i]);
-						}
 					if (is_playlist(command_line_path))
 						{
 						playlist_buf = g_list_prepend(playlist_buf, g_strdup(command_line_path));
@@ -1211,8 +1210,7 @@ int main (int argc, char *argv[])
 	if (initial_playlist_mode == PLAYLIST_MODE_SESSION && playlist_get_count() == 0 && !blank_startup)
 		{
 		gchar *session_file;
-		session_file = g_strconcat(homedir(), "/",
-			GQMPEG_RC_FILE_SESSION, NULL);
+		session_file = g_strconcat(homedir(), "/", GQMPEG_RC_FILE_SESSION, NULL);
 		if (isfile(session_file))
 			{
 			if (start_playing_on_startup)
