@@ -17,6 +17,7 @@
 #include "playlist-window.h"
 #include "preferences.h"
 #include "ui_help.h"
+#include "ui2_main.h"
 #include "ui_menu.h"
 
 
@@ -37,6 +38,17 @@ static GtkWidget *menu_preset(void)
 {
 	GtkWidget *menu;
 	gint i;
+	const gchar *match_path;
+
+	if (current_song_get_number() == -1)
+		{
+		match_path = current_song_get_path();
+		}
+	else
+		{
+		match_path = NULL;
+		}
+
 	menu = gtk_menu_new();
 	for (i=0; i<10; i++)
 		{
@@ -44,8 +56,14 @@ static GtkWidget *menu_preset(void)
 
 		if (preset_name[i] && preset_file[i])
 			{
+			gint match;
+
+			match = ((!match_path && playlist_pathname && strcmp(playlist_pathname, preset_file[i]) == 0) ||
+				 (match_path && strcmp(match_path, preset_file[i]) == 0) );
+
 			s = g_strdup_printf("%d - %s", i + 1, preset_name[i]);
-			menu_item_add(menu, s, GTK_SIGNAL_FUNC(load_preset_from_menu), GINT_TO_POINTER(i));
+			menu_item_add_check(menu, s, match,
+					    GTK_SIGNAL_FUNC(load_preset_from_menu), GINT_TO_POINTER(i));
 			g_free(s);
 			}
 		else
@@ -63,12 +81,26 @@ static GtkWidget *menu_preset(void)
 	return menu;
 }
 
+static void menu_editor_cb(GtkWidget *widget, gpointer data)
+{
+	UIData *ui = data;
+
+	btn_edit_skin_pressed(ui);
+}
+
 static void show_help(GtkWidget *widget, gpointer data)
 {
 	help_window_show("documentation");
 }
 
-GtkWidget *menu_main(void)
+static void close_window(GtkWidget *widget, gpointer data)
+{
+	UIData *ui = data;
+
+	if (ui != main_window) ui_close(ui);
+}
+
+GtkWidget *menu_main(UIData *ui)
 {
 	GtkWidget *menu;
 	GtkWidget *item;
@@ -122,11 +154,15 @@ GtkWidget *menu_main(void)
 
 	menu_item_add(menu, _("Run mixer..."), GTK_SIGNAL_FUNC(btn_mixer_pressed), NULL);
 	menu_item_add_divider(menu);
-	menu_item_add(menu, _("Skin Editor..."), GTK_SIGNAL_FUNC(btn_edit_skin_pressed), NULL);
-	menu_item_add(menu, _("Help..."), show_help, NULL);
+	menu_item_add(menu, _("Skin Editor..."), GTK_SIGNAL_FUNC(menu_editor_cb), ui);
+	menu_item_add(menu, _("Help..."), GTK_SIGNAL_FUNC(show_help), NULL);
 	menu_item_add(menu, _("About..."), GTK_SIGNAL_FUNC(show_about_info), NULL);
 	menu_item_add_divider(menu);
 	menu_item_add(menu, _("Iconify"), GTK_SIGNAL_FUNC(btn_iconify_pressed), NULL);
+	if (ui != main_window)
+		{
+		menu_item_add(menu, _("Close window"), GTK_SIGNAL_FUNC(close_window), ui);
+		}
 	menu_item_add(menu, _("Exit"), GTK_SIGNAL_FUNC(gqmpeg_exit), NULL);
 
 	return menu;
