@@ -20,6 +20,9 @@
                utility to read mpg header info
    ----------------------------------------------------------*/
 
+#define MPG_HDR_CHUNK_SIZE	1024
+#define MPG_HDR_MAX_READS	64
+
 unsigned int bitrates[3][3][15] = {
 	{
 	{0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448},
@@ -122,10 +125,11 @@ static Mpg_Data *parse_mpg_header(unsigned char *hdr, const gchar *path)
 
 Mpg_Data *get_mpg_header_info(const gchar *path)
 {
-	unsigned char buf[1024];
-	unsigned char *ptr;
+	Mpg_Data *md = NULL;
+	unsigned char buf[MPG_HDR_CHUNK_SIZE];
 	FILE *f = NULL;
-	gint c;
+	gint done = FALSE;
+	gint count = 0;
 
 	f = fopen(path, "r");
 	if (!f)
@@ -134,16 +138,36 @@ Mpg_Data *get_mpg_header_info(const gchar *path)
 		return NULL;
 		}
 
-	c = fread(buf, sizeof(unsigned char), 1024, f);
+	while (!done && !md && count < MPG_HDR_MAX_READS)
+		{
+		gint c;
+
+		c = fread(buf, sizeof(unsigned char), MPG_HDR_CHUNK_SIZE, f);
+		if (c < 3)
+			{
+			done = TRUE;
+			}
+		else
+			{
+			unsigned char *ptr = buf;
+
+			while (ptr < buf + c - 4 && !header_is_valid(ptr)) ptr++;
+			if (header_is_valid(ptr))
+				{
+				md = parse_mpg_header(ptr, path);
+				done = TRUE;
+				}
+			else if (count == 0)
+				{
+				/* if this is a file with an ID3v2 tag... keep going */
+				done = !(strncmp(buf, "ID3", 3) == 0);
+				}
+			}
+		count++;
+		}
 
 	fclose(f);
 
-	if (c < 3) return NULL;
-
-	ptr = buf;
-
-	while (ptr < buf + c - 4 && !header_is_valid(ptr)) ptr++;
-	if (!header_is_valid(ptr)) return NULL;
-
-	return parse_mpg_header(ptr, path);
+	return md;
 }
+
