@@ -58,7 +58,7 @@ static void save_tag_pressed(GtkWidget *w, gpointer data)
 	genre_text = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(s->genre_combo)->entry));
 
 	while (get_id3_genre(genre) && strcmp(get_id3_genre(genre), genre_text) != 0) genre++;
-	if (get_id3_genre(genre) == NULL) genre = 0;
+	if (get_id3_genre(genre) == NULL) genre = 255;
 
 	set_id3_tag(s->path, title, artist, album, year, comment, genre);
 	
@@ -87,13 +87,14 @@ static GtkWidget *add_combo_info(GtkWidget *vbox, const gchar *text, gint data,
 	GList *list = NULL;
 	gint i = 0;
 
-	while(get_id3_genre(i) != NULL)
+	while (get_id3_genre(i) != NULL)
 		{
 		gchar *g = get_id3_genre(i);
 		list = g_list_prepend(list, g);
 		i++;
 		}
 	list = g_list_sort(list, (GCompareFunc) sort_genre_list_cb);
+	list = g_list_prepend(list, "");
 
 	hbox = gtk_hbox_new(FALSE, 5);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
@@ -106,9 +107,16 @@ static GtkWidget *add_combo_info(GtkWidget *vbox, const gchar *text, gint data,
 	combo = gtk_combo_new();
 	gtk_combo_set_popdown_strings(GTK_COMBO(combo), list);
 	if (get_id3_genre(data) != NULL)
+		{
 		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo)->entry), get_id3_genre(data));
-	gtk_combo_set_value_in_list(GTK_COMBO(combo),
-				g_list_index(list, get_id3_genre(data)), FALSE);
+		gtk_combo_set_value_in_list(GTK_COMBO(combo),
+					    g_list_index(list, get_id3_genre(data)), FALSE);
+		}
+	else
+		{
+		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo)->entry), "");
+		gtk_combo_set_value_in_list(GTK_COMBO(combo), 0, FALSE);
+		}
 	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(combo)->entry), "changed",
 				GTK_SIGNAL_FUNC(data_changed_cb), pdata);
 	gtk_box_pack_end(GTK_BOX(hbox), combo, FALSE, FALSE, 0);
@@ -224,7 +232,7 @@ GtkWidget *mpg_create_info_window(const gchar *file)
 						    30, data_changed_cb, s);
 		s->year_entry = songinfo_add_entry(vbox1, _("Year"), d ? d->year : NULL,
 						   4, data_changed_cb, s);
-		s->genre_combo = add_combo_info(vbox1, _("Genre"), d ? d->genre : 0, s);
+		s->genre_combo = add_combo_info(vbox1, _("Genre"), d ? d->genre : 255, s);
 		s->comment_entry = songinfo_add_entry(vbox1, _("Comment"), d ? d->comment : NULL,
 						      30, data_changed_cb, s);
 
