@@ -54,6 +54,9 @@ static SongData *real_playlist_insert_item(const gchar *path, gint n, gint read_
 static gint real_playlist_remove_item(gint n);
 static gint real_playlist_move_item(gint s, gint t);
 
+static void playlist_background_reset(gint start);
+
+
 /*
  *-----------------------------------------------------------------------------
  * misc utils
@@ -541,7 +544,7 @@ static gint playlist_load_idle_cb(gpointer data)
 		}
 
 	new_time = gdk_time_get();
-	if (new_time - playlist_load_timer > 250)
+	if (new_time - playlist_load_timer > DISPLAY_LOADING_ANIMATE_TIME)
 		{
 		playlist_load_timer = new_time;
 		display_set_loading(TRUE);
@@ -604,7 +607,7 @@ gint playlist_load_start(const gchar *fn, gint append, gint strict, gint parse_b
 			{
 			/* check first line for valid playlist */
 			fgets(s_buf, 1024, playlist_load_fp);
-			if (strncmp(s_buf,"# GQmpeg",8) != 0)
+			if (strncmp(s_buf,"# GQ", 4) != 0)
 				{
 				/* we reach here, file is not a valid playlist */
 				fclose (playlist_load_fp);
@@ -969,6 +972,8 @@ static void real_playlist_init(void)
 
 static void real_playlist_clear(void)
 {
+	playlist_background_reset(FALSE);
+
         if (playlist)
                 {
                 GList *list;
@@ -1435,16 +1440,19 @@ gint playlist_remove(const gchar *path, gint n, gint all)
 	return ret;
 }
 
-void playlist_update_by_data(SongData *sd)
+void playlist_update_by_data_full(SongData *sd, gint force)
 {
 	gint n;
 
 	n = g_list_index(playlist, sd);
 	if (n < 0) return;
 
+	/* do not do it more than once when forcing */
+	if (force && sd->info_loaded) return;
+
 	playlist_length -= sd->length;
 
-	player_module_songdata_update(sd, read_file_information);
+	player_module_songdata_update(sd, (read_file_information || force));
 
 	if (current_song_get_data() == sd) display_set_song_text_info(-1, TRUE);
 
@@ -1454,6 +1462,11 @@ void playlist_update_by_data(SongData *sd)
 	display_total_time_changed();
 }
 
+void playlist_update_by_data(SongData *sd)
+{
+	playlist_update_by_data_full(sd, FALSE);
+}
+
 void playlist_update(gint n, const gchar *path)
 {
 	gint p = current_song_get_number();
@@ -1613,7 +1626,7 @@ void playlist_unset_flags(gint n, SongFlags flags)
 
 gint is_playlist(const gchar *path)
 {
-	if (strlen(path) > 8 && strncasecmp(path + (strlen(path) - 7), ".gqmpeg", 7) == 0) return TRUE;
+	if (strlen(path) > 7 && strncasecmp(path + (strlen(path) - 7), ".gqmpeg", 7) == 0) return TRUE;
 
 	return FALSE;
 }
@@ -1640,4 +1653,70 @@ void playlist_randomize(void)
 	display_set_song_number(current_song_get_number());
 }
 
+/*
+ *-----------------------------------------------------------------------------
+ * playlist on demand info background loading
+ *-----------------------------------------------------------------------------
+ */
+
+static GList *playlist_background_list = NULL;
+static gint playlist_background_id = -1;
+
+static gint playlist_background_cb(gpointer data)
+{
+	static guint32 old_time = 0;
+
+	SongData *sd;
+	guint32 new_time;
+
+	if (playlist_background_list == NULL)
+		{
+		playlist_background_id = -1;
+		display_set_loading(FALSE);
+		return FALSE;
+		}
+
+	new_time = gdk_time_get();
+	if (new_time - old_time > DISPLAY_LOADING_ANIMATE_TIME)
+		{
+		old_time = new_time;
+		display_set_loading(TRUE);
+		}
+
+	sd = playlist_background_list->data;
+	playlist_background_list = g_list_remove(playlist_background_list, sd);
+
+	/* this call does the pointer validation for us */
+	playlist_update_by_data_full(sd, TRUE);
+
+	return TRUE;
+}
+
+static void playlist_background_reset(gint start)
+{
+	if (!start && playlist_background_id != -1)
+		{
+		gtk_idle_remove(playlist_background_id);
+		playlist_background_id = -1;
+
+		g_list_free(playlist_background_list);
+		playlist_background_list = NULL;
+
+		display_set_loading(FALSE);
+		return;
+		}
+
+	if (start && playlist_background_id == -1)
+		{
+		playlist_background_id = gtk_idle_add(playlist_background_cb, NULL);
+		}
+}
+
+void playlist_update_background(SongData *sd)
+{
+	if (!sd || sd->info_loaded) return;
+
+	playlist_background_list = g_list_append(playlist_background_list, sd);
+	playlist_background_reset(TRUE);
+}
 
