@@ -15,6 +15,7 @@
 
 
 GList *path_list_sort(GList *list);
+GList *path_list_filter(GList *list, gint is_dir_list);
 
 void shuffle_list_create(gint start_with_current);
 void shuffle_list_destroy(void);
@@ -69,7 +70,9 @@ SongData *playlist_add(const gchar *path, gint read_info);
 SongData *playlist_insert(const gchar *path, gint n, gint read_info);
 gint playlist_move(gint s, gint t);
 gint playlist_remove(const gchar *path, gint n, gint all);
+void playlist_update_by_data_full(SongData *sd, gint force);
 void playlist_update_by_data(SongData *sd);
+void playlist_update_background(SongData *sd);
 void playlist_update(gint n, const gchar *path);
 void playlist_update_by_path(const gchar *path);
 void playlist_replace(const gchar *old_path, const gchar *new_path);
