@@ -14,6 +14,7 @@
 #include "preferences.h"
 
 #include "display.h"
+#include "display_flyby.h"
 #include "filelist.h"
 #include "ipc.h"
 #include "mixer.h"
@@ -67,6 +68,12 @@ static gint c_slik_transparency_force_a;
 
 static gint c_change_delay;
 
+static gint c_flyby_enabled;
+static gint c_flyby_location;
+
+static gint c_skinned_menus_enable;
+static gint c_slik_focus_enable;
+
 
 static void apply_configuration_window(void);
 static void create_config_window(gint start_tab);
@@ -158,6 +165,11 @@ static void initial_playlist_file_selected(GtkWidget *w, gpointer data)
 		c_initial_playlist_mode = PLAYLIST_MODE_FILE;
 }
 
+static void flyby_location_cb(GtkWidget *w, gpointer data)
+{
+	c_flyby_location = GPOINTER_TO_INT(data);
+}
+
 static void skin_item_selected(GtkWidget *clist, gint row, gint column,
 		GdkEventButton *bevent, gpointer data)
 {
@@ -275,6 +287,7 @@ static void ok_configuration_window(void)
 static void apply_configuration_window(void)
 {
 	gchar *buf = NULL;
+	const gchar *tmp;
 	gint i;
 	gint skin_changed = FALSE;
 
@@ -285,8 +298,8 @@ static void apply_configuration_window(void)
 		g_free(initial_playlist_pathname);
 		initial_playlist_pathname = NULL;
 		}
-	buf = gtk_entry_get_text(GTK_ENTRY(initial_playlist_entry));
-	if (buf&& strlen(buf) > 0) initial_playlist_pathname = g_strdup(buf);
+	tmp = gtk_entry_get_text(GTK_ENTRY(initial_playlist_entry));
+	if (tmp && strlen(tmp) > 0) initial_playlist_pathname = g_strdup(tmp);
 
 	if ( slik_double_size != c_slik_double_size || 
 		show_extra_winamp_info != c_show_extra_winamp_info) skin_changed = TRUE;
@@ -351,8 +364,8 @@ static void apply_configuration_window(void)
 		g_free(mixer_command);
 		mixer_command = NULL;
 		}
-	buf = gtk_entry_get_text(GTK_ENTRY(mixer_entry));
-	if (buf && strlen(buf) > 0) mixer_command = g_strdup(buf);
+	tmp = gtk_entry_get_text(GTK_ENTRY(mixer_entry));
+	if (tmp && strlen(tmp) > 0) mixer_command = g_strdup(tmp);
 
 	start_playing_on_startup = c_start_playing_on_startup;
 	show_playlist_on_startup = c_show_playlist_on_startup;
@@ -363,6 +376,9 @@ static void apply_configuration_window(void)
 	slik_remember_position = c_slik_remember_position;
 	wm_decorations = c_wm_decorations;
 
+	skinned_menus_enable = c_skinned_menus_enable;
+	slik_focus_enable = c_slik_focus_enable;
+
 	play_presets = c_play_presets;
 	drop_single_to_playlist = c_drop_single_to_playlist;
 	play_next_on_error = c_play_next_on_error;
@@ -413,6 +429,10 @@ static void apply_configuration_window(void)
 		}
 
 	change_delay = c_change_delay;
+
+	flyby_enabled = c_flyby_enabled;
+	flyby_location = c_flyby_location;
+
 	prev_includes_current = c_prev_includes_current;
 
 	for (i=0; i<10; i++)
@@ -420,15 +440,15 @@ static void apply_configuration_window(void)
 		if (preset_name[i]) g_free(preset_name[i]);
 		if (preset_file[i]) g_free(preset_file[i]);
 
-		buf = gtk_entry_get_text(GTK_ENTRY(preset_name_entry[i]));
-		if (strlen(buf) > 0)
-			preset_name[i] = g_strdup(buf);
+		tmp = gtk_entry_get_text(GTK_ENTRY(preset_name_entry[i]));
+		if (strlen(tmp) > 0)
+			preset_name[i] = g_strdup(tmp);
 		else
 			preset_name[i] = NULL;
 
-		buf = gtk_entry_get_text(GTK_ENTRY(preset_file_entry[i]));
-		if (strlen(buf) > 0)
-			preset_file[i] = g_strdup(buf);
+		tmp = gtk_entry_get_text(GTK_ENTRY(preset_file_entry[i]));
+		if (strlen(tmp) > 0)
+			preset_file[i] = g_strdup(tmp);
 		else
 			preset_file[i] = NULL;
 		}
@@ -627,12 +647,13 @@ static void create_config_window(gint start_tab)
 
 	lh = 0;
 	cnt = 0;
-	while(list)
+	while (list)
 		{
 		gchar *dev_text = list->data;
 	        item = gtk_menu_item_new_with_label(dev_text);
-	        gtk_signal_connect (GTK_OBJECT (item), "activate", (GtkSignalFunc) mixer_device_id_cb, dev_text);
-        	gtk_menu_append (GTK_MENU (menu), item);
+	        gtk_signal_connect(GTK_OBJECT(item), "activate",
+				   GTK_SIGNAL_FUNC(mixer_device_id_cb), dev_text);
+		gtk_menu_append(GTK_MENU(menu), item);
 		gtk_widget_show(item);
 		if (get_mixer_device_id(dev_text) == mixer_device_id) lh = cnt;
 		cnt++;
@@ -641,10 +662,10 @@ static void create_config_window(gint start_tab)
 
 	g_list_free(devices);
 
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu), menu);
-        gtk_option_menu_set_history (GTK_OPTION_MENU (omenu), lh);
-        gtk_box_pack_start( GTK_BOX(hbox), omenu, TRUE, TRUE, 5);
-        gtk_widget_show (omenu);
+	gtk_option_menu_set_menu(GTK_OPTION_MENU(omenu), menu);
+	gtk_option_menu_set_history(GTK_OPTION_MENU(omenu), lh);
+	gtk_box_pack_start(GTK_BOX(hbox), omenu, TRUE, TRUE, 5);
+	gtk_widget_show(omenu);
 
 	button = checkbox_new(_("First press of prev restarts song when playing"), prev_includes_current, &c_prev_includes_current);
 	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
@@ -665,7 +686,8 @@ static void create_config_window(gint start_tab)
 	c_change_delay = change_delay;
 	adj = gtk_adjustment_new((gfloat)change_delay, 0.0, 300.0, 1.0, 1.0, 1.0);
 	spin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1.0, 0);
-	gtk_signal_connect(adj, "value_changed", spin_cb, &c_change_delay);
+	gtk_signal_connect(adj, "value_changed",
+			   GTK_SIGNAL_FUNC(spin_cb), &c_change_delay);
 	gtk_box_pack_start(GTK_BOX(hbox), spin, FALSE, FALSE, 0);
 	gtk_widget_show(spin);
 
@@ -819,6 +841,45 @@ static void create_config_window(gint start_tab)
 	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
 	gtk_widget_show(button);
 
+	frame = gtk_frame_new(_("Flybys"));
+	gtk_container_border_width (GTK_CONTAINER (frame), 5);
+	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
+	gtk_widget_show(frame);
+
+	vbox1 = gtk_vbox_new (FALSE, 0);
+	gtk_container_add (GTK_CONTAINER(frame), vbox1);
+	gtk_widget_show(vbox1);
+
+	hbox = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
+	gtk_widget_show(hbox);
+
+	button = checkbox_new(_("Enable flybys"), flyby_enabled, &c_flyby_enabled);
+	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+	gtk_widget_show(button);
+
+	c_flyby_location = flyby_location;
+
+	omenu = gtk_option_menu_new();
+	menu = gtk_menu_new();
+
+	for (i = 0; i < 4; i++)
+		{
+		const gchar *loc_text;
+
+		loc_text = display_flyby_location_to_text(i);
+		item = gtk_menu_item_new_with_label(loc_text);
+		gtk_signal_connect(GTK_OBJECT(item), "activate",
+				   GTK_SIGNAL_FUNC(flyby_location_cb), GINT_TO_POINTER(i));
+		gtk_menu_append(GTK_MENU(menu), item);
+		gtk_widget_show(item);
+		}
+
+	gtk_option_menu_set_menu(GTK_OPTION_MENU(omenu), menu);
+	gtk_option_menu_set_history(GTK_OPTION_MENU(omenu), flyby_location);
+	gtk_box_pack_start(GTK_BOX(hbox), omenu, TRUE, TRUE, 5);
+	gtk_widget_show(omenu);
+
 	/* preset tab */
 
 	frame = gtk_frame_new(NULL);
@@ -853,8 +914,8 @@ static void create_config_window(gint start_tab)
 		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 		gtk_widget_show(hbox);
 
-		sprintf(s,"%d",i + 1);
-                label = gtk_label_new(s);
+		snprintf(s, sizeof(s), "%d", i + 1);
+		label = gtk_label_new(s);
 		gtk_table_attach_defaults(GTK_TABLE (table),label,0,1,i+1,i+2);
 		gtk_widget_show(label);
 
@@ -986,6 +1047,14 @@ static void create_config_window(gint start_tab)
         gtk_box_pack_start(GTK_BOX(hbox), hscale, TRUE, TRUE, 0);
         gtk_widget_show(hscale);
 
+	button = checkbox_new(_("Enable menu skins"), skinned_menus_enable, &c_skinned_menus_enable);
+	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
+	gtk_widget_show(button);
+
+	button = checkbox_new(_("Enable keyboard focus"), slik_focus_enable, &c_slik_focus_enable);
+	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
+	gtk_widget_show(button);
+
 	sep = gtk_hseparator_new();
 	gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 0);
 	gtk_widget_show(sep);
