@@ -27,6 +27,9 @@
 #include "ui2_slider.h"
 
 
+#include <gdk/gdkkeysyms.h> /* for key values */
+
+
 #define UI2_LIST_SCROLL_DELAY 200
 #define UI2_LIST_SCROLL_DELAY_FAST 67
 #define UI2_LIST_SCROLL_SPEEDUP 4
@@ -46,6 +49,10 @@ struct _ListCallbackData
 	gpointer row_click_data;
 	gpointer row_select_data;
 	gpointer row_move_data;
+
+	/* only use for menu mode */
+	gint (*menu_move_func)(ListData *list, const gchar *key, gint row, gint activated, gint up, gpointer data);
+	gpointer menu_move_data;
 };
 
 
@@ -68,6 +75,8 @@ ListData *list_new(GdkPixbuf *pb, gint x, gint y, gint w, gint h, gint sizeable,
 {
 	ListData *ld;
 
+	list_type_init();
+
 	if (!pb) return NULL;
 
 	ld = g_new0(ListData, 1);
@@ -100,11 +109,14 @@ ListData *list_new(GdkPixbuf *pb, gint x, gint y, gint w, gint h, gint sizeable,
 	ld->column_real_widths = g_new0(gint, columns);
 	ld->column_keys = g_new0(gchar *, columns);
 
+	ld->columns_right_justify = FALSE;
+
 	ld->overlay = util_size_pixbuf(pb, TRUE);
 
 	ld->timeout_id = -1;
 	ld->row_prelight = -1;
 	ld->row_press = -1;
+	ld->focus_row = -1;
 	
 	return ld;
 }
@@ -124,6 +136,12 @@ void list_set_column_attributes(ListData *ld, gint column, gint length, ListColu
 	if (key && strcmp(key, "flags") == 0) ld->flag_column = column;
 }
 
+void list_set_column_justify(ListData *ld, gint justify_right)
+{
+	if (!ld) return;
+	ld->columns_right_justify = justify_right;
+}
+
 void list_image_row(ListData *ld, GdkPixbuf *pb, gint has_press, gint has_prelight,
 		    gint border_left, gint border_right, gint stretch, GdkPixbuf *divider)
 {
@@ -224,6 +242,7 @@ static ListRowData *list_row_new(guint columns)
 	rd = g_new0(ListRowData, 1);
 	rd->cells = columns;
 	rd->text = g_new0(gchar*, columns + 1);
+	rd->sensitive = TRUE;
 	rd->flag_mask = -1;
 
 	return rd;
@@ -307,9 +326,12 @@ void list_row_column_set_text(ListData *ld, ListRowData *rd, const gchar *column
 
 void list_row_set_flag(ListRowData *rd, gint flag)
 {
-	if (!rd) return;
+	if (rd) rd->flag_mask = flag;
+}
 
-	rd->flag_mask = flag;
+void list_row_set_sensitive(ListRowData *rd, gint sensitive)
+{
+	if (rd) rd->sensitive = sensitive;
 }
 
 /*
@@ -359,12 +381,22 @@ static void list_sync(ListData *ld)
 		{
 		gint c;
 		gint s;
+		gint p;
 
 		s = w;
 
+		if (ld->columns_right_justify)
+			{
+			p = (ld->column_count - 1) - i;
+			}
+		else
+			{
+			p = i;
+			}
+
 		if (w >= area)
 			{
-			ld->column_real_widths[i] = 0;
+			ld->column_real_widths[p] = 0;
 			continue;
 			}
 
@@ -375,20 +407,20 @@ static void list_sync(ListData *ld)
 			}
 		else
 			{
-			if (ld->column_flags[i] & UI_LIST_COLUMN_SIZE_PROPORTIONAL)
+			if (ld->column_flags[p] & UI_LIST_COLUMN_SIZE_PROPORTIONAL)
 				{
-				c = ((float)ld->column_widths[i] / 100.0 * ld->region_width) - ld->divider_width;
+				c = ((float)ld->column_widths[p] / 100.0 * ld->region_width) - ld->divider_width;
 				if (c < 1) c = 1;
 				}
 			else
 				{
-				c = ld->column_widths[i];
+				c = ld->column_widths[p];
 				}
 			w += c + ld->divider_width;
 			}
 		if (w >= area) c = area - s;
-		ld->column_real_widths[i] = c;
-		if (debug_mode) printf("column %d with width of %d (%d)\n", i, c, ld->column_widths[i]);
+		ld->column_real_widths[p] = c;
+		if (debug_mode) printf("column %d with width of %d (%d)\n", p, c, ld->column_widths[p]);
 		}
 
 	/* vertical centering offsets */
@@ -428,6 +460,15 @@ static gint list_length_sync(ListData *ld, gint length, gint force)
 	if (ld->row_count == length && !force) return FALSE;
 	if (!force) ld->row_count = length;
 
+	if (ld->focus_row >= ld->row_count)
+		{
+		ld->focus_row = ld->row_count - 1;
+		}
+	else if (ld->focus_row < 0 && ld->row_count > 0)
+		{
+		ld->focus_row = 0;
+		}
+
 	if (list_scroll_clamp(ld))
 		{
 		gint i;
@@ -531,7 +572,7 @@ static const gchar iso_ascii[]=
 };
 
 static void list_draw_text(ListData *ld, GdkPixbuf *pb, const gchar *text, gint x, gint y,
-			   gint w, gint right_justify)
+			   gint w, gint right_justify, gint alpha)
 {
 	gint l;
 	gint p;
@@ -588,7 +629,7 @@ static void list_draw_text(ListData *ld, GdkPixbuf *pb, const gchar *text, gint
 
 		pixbuf_copy_area_alpha(ld->text_overlay, px, py,
 				       pb, x + p, y,
-				       ld->char_width, ld->char_height, 255);
+				       ld->char_width, ld->char_height, alpha);
 		i++;
 		p += ld->char_width;
 		}
@@ -599,25 +640,29 @@ static void list_draw_cell(ListData *ld, GdkPixbuf *pb, gint x, gint y, gint row
 	ListRowData *rd;
 	const gchar *text;
 	gint w;
+	gint alpha;
 
 	rd = g_list_nth_data(ld->row_list, row);
 	if (!rd) return;
 
+	alpha = (rd->sensitive) ? 255 : 86;
+
 	w = ld->column_real_widths[column];
 	text = list_row_text_get(rd, column);
 	if (text) list_draw_text(ld, pb, text, ld->x + x, ld->y + y + ld->text_voff, w,
-				 (ld->column_flags[column] & UI_LIST_COLUMN_JUSTIFY_RIGHT));
+				 (ld->column_flags[column] & UI_LIST_COLUMN_JUSTIFY_RIGHT),
+				 alpha);
 	if (divider && ld->divider_overlay)
 		{
 		pixbuf_copy_area_alpha(ld->divider_overlay, 0, 0,
 				       pb, ld->x + x + w, ld->y + y + ld->divider_voff,
-				       ld->divider_width, ld->divider_height, 255);
+				       ld->divider_width, ld->divider_height, alpha);
 		}
 	if (column == ld->flag_column && ld->flag_overlay && rd->flag_mask >= 0 && rd->flag_mask < ld->flag_sections)
 		{
 		pixbuf_copy_area_alpha(ld->flag_overlay, 0, rd->flag_mask * ld->flag_height,
 				       pb, ld->x + x, ld->y + y + ld->flag_voff,
-				       MIN(ld->flag_width, w), MIN(ld->flag_height, ld->row_height), 255);
+				       MIN(ld->flag_width, w), MIN(ld->flag_height, ld->row_height), alpha);
 		}
 }
 
@@ -646,6 +691,11 @@ static void list_draw_row(ListData *ld, GdkPixbuf *pb, UIData *ui, gint row, gin
 			 FALSE);
 	list_draw_row_image(ld, pb, 0, x, y);
 
+	if (ld->row_has_press && row == ld->row_press - ld->row_start)
+		{
+		list_draw_row_image(ld, pb, ld->row_height, x, y);
+		}
+
 	x += ld->row_border_left;
 	i = 0;
 	while(i < ld->column_count && ld->column_real_widths[i] > 0)
@@ -660,11 +710,7 @@ static void list_draw_row(ListData *ld, GdkPixbuf *pb, UIData *ui, gint row, gin
 
 	x = ld->border_left;
 
-	if (ld->row_has_press && row == ld->row_press - ld->row_start)
-		{
-		list_draw_row_image(ld, pb, ld->row_height, x, y);
-		}
-	else if (ld->row_has_prelight && row == ld->row_prelight - ld->row_start)
+	if (ld->row_has_prelight && row == ld->row_prelight - ld->row_start)
 		{
 		list_draw_row_image(ld, pb, ld->row_height * (ld->row_has_press + 1), x, y);
 		}
@@ -672,7 +718,7 @@ static void list_draw_row(ListData *ld, GdkPixbuf *pb, UIData *ui, gint row, gin
 	if (render && ui)
 		{
 		ui_display_render_area(ui, ld->x + ld->border_left, ld->y + y,
-				       ld->region_width, ld->row_height);
+				       ld->region_width, ld->row_height, ld->wd);
 		}
 }
 
@@ -694,7 +740,7 @@ static void list_draw_empty_rows(ListData *ld, GdkPixbuf *pb, UIData *ui, gint r
 	if (render && ui)
 		{
 		ui_display_render_area(ui, ld->x + ld->border_left, ld->y + ld->border_top + y,
-				      ld->region_width, ld->region_height - y);
+				      ld->region_width, ld->region_height - y, ld->wd);
 		}
 }
 
@@ -734,7 +780,7 @@ static void list_draw_rows(ListData *ld, GdkPixbuf *pb, UIData *ui, gint start,
 	if (render && ui)
 		{
 		ui_display_render_area(ui, ld->x + ld->border_left, ld->y + ld->border_top + start * ld->row_height,
-				       ld->region_width, (end - start + 1) * ld->row_height);
+				       ld->region_width, (end - start + 1) * ld->row_height, ld->wd);
 		}
 }
 
@@ -752,7 +798,7 @@ static void list_draw_all(ListData *ld, GdkPixbuf *pb, UIData *ui)
 	list_draw_border(ld, pb);
 	list_draw_all_rows(ld, pb, ui, FALSE);
 
-	ui_display_render_area(ui, ld->x, ld->y, ld->width, ld->height);
+	ui_display_render_area(ui, ld->x, ld->y, ld->width, ld->height, ld->wd);
 }
 
 /*
@@ -813,6 +859,11 @@ static void list_row_scroll_by_row(ListData *ld, GdkPixbuf *pb, UIData *ui, cons
 		list_row_sync_all(ld, ui, key, NULL);
 		}
 
+	if (ld->row_prelight >= 0)
+		{
+		ld->row_prelight += ld->row_start - old_s;
+		}
+
 	list_draw_rows(ld, pb, ui, 0, ld->row_end - ld->row_start, TRUE);
 }
 
@@ -929,15 +980,30 @@ static void list_reset(gpointer data, const gchar *key, GdkPixbuf *pb, UIData *u
 static void list_motion(gpointer data, const gchar *key, gint x, gint y, GdkPixbuf *pb, UIData *ui)
 {
 	ListData *ld = data;
+	gint old_prelight;
 
 	if (ld->pressed)
 		{
 		gint row;
 
 		row = list_row_find_proximity(ld, x, y);
-		if (row == ld->press_row)
+		if (row == ld->press_row || ld->menu_style_select)
 			{
+			gint changed = (row != ld->row_press);
+
 			list_row_press_set(ld, pb, ui, row);
+
+			if (changed && ld->menu_style_select)
+				{
+				ListCallbackData *cd;
+	
+				cd = ui_get_registered_callbacks(ui, key, type_id);
+				if (cd && cd->menu_move_func)
+					{
+					cd->menu_move_func(ld, key, row, FALSE, FALSE, cd->menu_move_data);
+					}
+				}
+
 			}
 		else
 			{
@@ -947,6 +1013,8 @@ static void list_motion(gpointer data, const gchar *key, gint x, gint y, GdkPixb
 		return;
 		}
 
+	old_prelight = ld->row_prelight;
+
 	if (list_test_proximity(ld, x, y))
 		{
 		gint row;
@@ -958,6 +1026,17 @@ static void list_motion(gpointer data, const gchar *key, gint x, gint y, GdkPixb
 		{
 		list_row_prelight_set(ld, pb, ui, -1);
 		}
+
+	if (old_prelight != ld->row_prelight && ld->menu_style_select)
+		{
+		ListCallbackData *cd;
+	
+		cd = ui_get_registered_callbacks(ui, key, type_id);
+		if (cd && cd->menu_move_func)
+			{
+			cd->menu_move_func(ld, key, ld->row_prelight, FALSE, FALSE, cd->menu_move_data);
+			}
+		}
 }
 
 static gint list_press(gpointer data, const gchar *key, gint x, gint y, GdkPixbuf *pb, UIData *ui)
@@ -989,7 +1068,7 @@ static gint list_press(gpointer data, const gchar *key, gint x, gint y, GdkPixbu
 				{
 				if (cd && cd->row_select_func)
 					{
-					cd->row_select_func(ld, key, row, NULL, cd->row_click_data);
+					cd->row_select_func(ld, key, row, NULL, cd->row_select_data);
 					}
 				t = 0;
 				}
@@ -999,6 +1078,30 @@ static gint list_press(gpointer data, const gchar *key, gint x, gint y, GdkPixbu
 			return FALSE;
 			}
 
+		if (row != ld->focus_row)
+			{
+			gint old_row;
+
+			old_row = ld->focus_row;
+			ld->focus_row = row;
+			
+			if (old_row >= 0 && old_row >= ld->row_start && old_row <= ld->row_end)
+				{
+				list_draw_row(ld, pb, ui, old_row - ld->row_start, TRUE);
+				}
+
+			if (ld->menu_style_select)
+				{
+				ListCallbackData *cd;
+	
+				cd = ui_get_registered_callbacks(ui, key, type_id);
+				if (cd && cd->menu_move_func)
+					{
+					cd->menu_move_func(ld, key, row, FALSE, FALSE, cd->menu_move_data);
+					}
+				}
+			}
+
 		ld->press_row = row;
 		list_row_press_set(ld, pb, ui, row);
 		ld->click_time = t;
@@ -1064,6 +1167,162 @@ static void list_back_set(gpointer data, GdkPixbuf *pb)
 				      ld->stretch, 255);
 }
 
+static gint list_key_event_cb(gpointer widget, const gchar *key, GdkEventKey *event, GdkPixbuf *pb, UIData *ui)
+{
+	ListData *ld = widget;
+	gint ret = FALSE;
+	gint new_row;
+	gint old_row;
+
+	old_row = new_row = ld->focus_row;
+
+	switch (event->keyval)
+		{
+		case GDK_space:
+			if (new_row >= 0)
+				{
+				ListCallbackData *cd;
+
+				cd = ui_get_registered_callbacks(ui, key, type_id);
+				if (cd && cd->row_click_func)
+					{
+					cd->row_click_func(ld, key, new_row, NULL, 0, cd->row_click_data);
+					}
+				}
+			ret = TRUE;
+			break;
+		case GDK_Return:
+		case GDK_KP_Enter:
+			if (new_row >= 0)
+				{
+				ListCallbackData *cd;
+
+				cd = ui_get_registered_callbacks(ui, key, type_id);
+				if (cd && cd->row_select_func)
+					{
+					cd->row_select_func(ld, key, new_row, NULL, cd->row_select_data);
+					}
+				}
+			ret = TRUE;
+			break;
+		case GDK_Left:
+		case GDK_KP_Left:
+		case GDK_Right:
+		case GDK_KP_Right:
+			if (ld->menu_style_select)
+				{
+				ListCallbackData *cd;
+
+				cd = ui_get_registered_callbacks(ui, key, type_id);
+				if (cd && cd->menu_move_func)
+					{
+					gint up;
+
+					up = (event->keyval == GDK_Right || event->keyval == GDK_KP_Right);
+					ret = cd->menu_move_func(ld, key, new_row, TRUE, up, cd->menu_move_data);
+					}
+				}
+			break;
+		case GDK_Up:
+		case GDK_KP_Up:
+			new_row--;
+			break;
+		case GDK_Down:
+		case GDK_KP_Down:
+			new_row++;
+			break;
+		case GDK_Page_Up:
+		case GDK_KP_Page_Up:
+			new_row -= MAX((list_rows_visible(ld) - 1), 1);
+			ret = TRUE;
+			break;
+		case GDK_Page_Down:
+		case GDK_KP_Page_Down:
+			new_row += MAX((list_rows_visible(ld) - 1), 1);
+			ret = TRUE;
+			break;
+		case GDK_Home:
+		case GDK_KP_Home:
+			new_row = 0;
+			ret = TRUE;
+			break;
+		case GDK_End:
+		case GDK_KP_End:
+			new_row = ld->row_count - 1;
+			ret = TRUE;
+			break;
+		default:
+			break;
+		}
+
+	if (new_row > ld->row_count - 1)
+		{
+		new_row = ld->row_count - 1;
+		}
+	else if (new_row < 0 && ld->row_count > 0)
+		{
+		new_row = 0;
+		}
+
+	ret = (ret || (ld->focus_row != new_row));
+	ld->focus_row = new_row;
+
+	if (new_row >= 0)
+		{
+		if (new_row < ld->row_start)
+			{
+			list_row_scroll_by_row(ld, pb, ui, key, new_row - ld->row_start);
+			}
+		else if (new_row > ld->row_end)
+			{
+			list_row_scroll_by_row(ld, pb, ui, key, new_row - ld->row_end);
+			}
+		else
+			{
+			/* scrolling handles redraws, no scroll do it ourselves */
+			if (old_row >= ld->row_start && old_row <= ld->row_end)
+				{
+				list_draw_row(ld, pb, ui, old_row - ld->row_start, TRUE);
+				}
+			list_draw_row(ld, pb, ui, new_row - ld->row_start, TRUE);
+			}
+
+		if (ld->menu_style_select)
+			{
+			ListCallbackData *cd;
+
+			cd = ui_get_registered_callbacks(ui, key, type_id);
+			if (cd && cd->menu_move_func)
+				{
+				cd->menu_move_func(ld, key, new_row, FALSE, FALSE, cd->menu_move_data);
+				}
+			}
+		}
+
+	return ret;
+}
+
+static gint list_focus_draw_cb(gpointer widget, const gchar *key,
+			       gint x, gint y, gint w, gint h, GdkPixbuf *pb, UIData *ui)
+{
+	ListData *ld = widget;
+	gint rx, ry, rw, rh;
+
+	if (ld->focus_row < 0 ||
+	    ld->focus_row < ld->row_start ||
+	    ld->focus_row > ld->row_end) return TRUE;
+
+	rx = ld->border_left;
+	ry = (ld->focus_row - ld->row_start) * ld->row_height + ld->border_top;
+	rw = ld->region_width;
+	rh = ld->row_height;
+
+	ui_display_draw_focus(ui, pb, ld->x + rx, ld->y + ry, rw, rh,
+			      x, y, w, h, NULL);
+
+	return TRUE;
+}
+
 static gint list_get_geometry(gpointer widget, gint *x, gint *y, gint *w, gint *h)
 {
 	ListData *ld = widget;
@@ -1109,6 +1368,7 @@ static WidgetData *list_parse(SkinData *skin, GList *list, const gchar *skin_dir
 	gint border_left;
 	gint center_stretch;
 	gint columns;
+	gint columns_right_justify;
 	gchar *filename;
 	gchar *row_filename;
 	gchar *divider_filename;
@@ -1166,6 +1426,7 @@ static WidgetData *list_parse(SkinData *skin, GList *list, const gchar *skin_dir
 	if (!key_list_read_int(list, "flag_sections", &flag_sections)) flag_sections = 1;
 	if (!key_list_read_int(list, "flag_column", &flag_column)) flag_column = 0;
 
+	columns_right_justify = key_list_read_bool(list, "columns_right_justify");
 
 	text_extended = key_list_read_bool(list, "text_extended");
 
@@ -1181,6 +1442,8 @@ static WidgetData *list_parse(SkinData *skin, GList *list, const gchar *skin_dir
 		list_image_text(ld, gdk_pixbuf_new_from_file(text_filename), text_extended);
 		if (flag_filename) list_image_row_flag(ld, gdk_pixbuf_new_from_file(flag_filename), flag_sections, flag_column);
 
+		list_set_column_justify(ld, columns_right_justify);
+
 		for (i = 0; i < columns; i++)
 			{
 			gchar *buf;
@@ -1260,6 +1523,8 @@ RegisterData *list_register_key(const gchar *key, UIData *ui,
 {
 	ListCallbackData *cd;
 
+	list_type_init();
+
         cd = g_new0(ListCallbackData, 1);
 
 	cd->length_request_func = length_request_func;
@@ -1274,9 +1539,27 @@ RegisterData *list_register_key(const gchar *key, UIData *ui,
 	cd->row_select_data = row_select_data;
 	cd->row_move_data = row_move_data;
 
+	cd ->menu_move_func = NULL;
+	cd ->menu_move_data = NULL;
+
 	return ui_register_key(ui, key, type_id, cd, sizeof(ListCallbackData));
 }
 
+void list_register_menu_funcs(const gchar *key, UIData *ui,
+			      gint (*func)(ListData *list, const gchar *key, gint row, gint activated, gint up, gpointer data),
+			      gpointer data)
+{
+	ListCallbackData *cd;
+
+	cd = ui_get_registered_callbacks(ui, key, type_id);
+
+	if (cd)
+		{
+		cd ->menu_move_func = func;
+		cd ->menu_move_data = data;
+		}
+}
+
 /*
  *-----------------------------
  * app funcs
@@ -1384,7 +1667,7 @@ static void list_refresh_cb(WidgetData *wd, gpointer data, GdkPixbuf *pb, UIData
 	list_draw_all_rows(ld, pb, ui, FALSE);
 
 	ui_display_render_area(ui, ld->x + ld->border_left, ld->y + ld->border_top,
-			       ld->region_width, ld->region_height);
+			       ld->region_width, ld->region_height, ld->wd);
 }
 
 gint list_refresh(const gchar *key, UIData *ui)
@@ -1424,6 +1707,11 @@ gint list_scroll_to(const gchar *key, UIData *ui, gint row)
 	return skin_widget_for_each_key(ui, key, type_id, list_scroll_to_cb, GINT_TO_POINTER(row));
 }
 
+void list_set_menu_style(ListData *ld, gint menu_style)
+{
+	if (ld) ld->menu_style_select = menu_style;
+}
+
 /*
  *-----------------------------
  * internal signals
@@ -1578,11 +1866,19 @@ static void list_init(gpointer widget, const gchar *key, UIData *ui)
 static void list_scroll_update_widgets(ListData *ld)
 {
 	gchar *buf;
+	gint total;
+	gint vis;
 
 	if (!ld->ui) return;
 
+	vis = list_rows_visible(ld);
+	total = ld->row_count - vis;
+
 	buf = g_strdup_printf("list_%s_scroll", ld->wd->key);
 	slider_value_set(buf, ld->ui, list_scroll_position(ld));
+	slider_step_size_set(buf, ld->ui,
+			     (total > 0) ? (1.0 / (float)total) : 0.1,
+			     (vis > 1) ? vis - 1 : vis);
 	g_free(buf);
 }
 
@@ -1614,6 +1910,9 @@ void list_type_init(void)
 	od->func_back = list_back_set;
 	od->func_free = list_free_cb;
 
+	od->func_focus_key_event = list_key_event_cb;
+	od->func_focus_draw = list_focus_draw_cb;
+
 	od->func_get_geometry = list_get_geometry;
 	od->func_set_coord = list_set_coord;
 	od->func_set_size = list_set_size;
@@ -1624,6 +1923,3 @@ void list_type_init(void)
 	list_type_init_edit(od);
 }
 
-
-
-
