@@ -42,6 +42,8 @@ DigitData *digit_new(GdkPixbuf *pb)
 {
 	DigitData *digit;
 
+	digit_type_init();
+
 	if (!pb) return NULL;
 
 	digit = g_new0(DigitData, 1);
@@ -179,6 +181,15 @@ WidgetData *digit_register(SkinData *skin, DigitData *digit, const gchar *key, c
 	return skin_register_widget(skin, key, text_id, type_id_digit, digit);
 }
 
+WidgetData *digit_register_to_skin(const gchar *key, gchar **data,
+				   SkinData *skin, const gchar *text_id)
+{
+	DigitData *digit;
+
+	digit = digit_new_from_data(data);
+	return digit_register(skin, digit, key, text_id);
+}
+
 /*
  *-----------------------------
  * init
@@ -233,10 +244,14 @@ static WidgetType type_id_number = -1;
  *-----------------------------
  */
 
-NumberData *number_new(DigitData *digit, gint length, gint zeros, gint centered, gint x, gint y)
+NumberData *number_new(DigitData *digit, gint x, gint y,
+		       gint length, gint zeros, gint centered)
 {
 	NumberData *number;
 
+	digit_type_init();
+	number_type_init();
+
 	if (!digit || length < 1) return NULL;
 
 	number = g_new0(NumberData, 1);
@@ -346,7 +361,7 @@ static void number_draw_real(NumberData *number, gint n, GdkPixbuf *pb, UIData *
 			}
 		}
 
-	ui_display_render_area(ui, number->x, number->y, digit->width * number->length, digit->height);
+	ui_display_render_area(ui, number->x, number->y, digit->width * number->length, digit->height, NULL);
 }
 
 /*
@@ -458,7 +473,7 @@ static WidgetData *number_parse(SkinData *skin, GList *list, const gchar *skin_d
 			}
 		}
 
-	number = number_new(digit, length, zeros, center, x, y);
+	number = number_new(digit, x, y, length, zeros, center);
 
 	if (number)
 		{
@@ -488,11 +503,24 @@ WidgetData *number_register(SkinData *skin, NumberData *number, const gchar *key
 	return skin_register_widget(skin, key, text_id, type_id_number, number);
 }
 
+WidgetData *number_register_to_skin(const gchar *key, DigitData *digit, gint x, gint y,
+				    gint length, gint zeros, gint centered,
+				    SkinData *skin, const gchar *text_id)
+{
+	NumberData *number;
+
+	number = number_new(digit, x, y, length, zeros, centered);
+	return number_register(skin, number, key, text_id);
+}
+
 RegisterData *number_register_key(const gchar *key, UIData *ui,
 				  gint (*status_get_func)(NumberData *number, const gchar *key, gpointer data), gpointer status_get_data)
 {
 	NumberCallbackData *cd;
 
+	digit_type_init();
+	number_type_init();
+
 	cd = g_new0(NumberCallbackData, 1);
 
 	cd->status_get_func = status_get_func;
@@ -552,6 +580,8 @@ void number_type_init(void)
 
 	if (type_id_number != -1) return;
 
+	digit_type_init();
+
 	od = ui_widget_type_new("number");
 	type_id_number = od->type;
 
@@ -570,3 +600,4 @@ void number_type_init(void)
 
 
 
+
