@@ -17,6 +17,7 @@
 
 #include "ui2_button.h"
 #include "ui2_display.h"
+#include "ui2_main.h"
 #include "ui2_util.h"
 #include "ui2_widget.h"
 #include "ui_pixbuf_ops.h"
@@ -177,17 +178,17 @@ WidgetData *skin_widget_get_by_text_id(SkinData *skin, const gchar *text_id)
 	return NULL;
 }
 
-gint skin_widget_for_each_key(UIData *ui, const gchar *key, WidgetType type,
-			      void (*func)(WidgetData *wd, gpointer data, GdkPixbuf *pb, UIData *ui),
-			      gpointer data)
+static gint skin_widget_for_each_do(UIData *ui, const gchar *key, WidgetType type,
+				    void (*func)(WidgetData *wd, gpointer data, GdkPixbuf *pb, UIData *ui),
+				    gpointer data)
 {
 	gint count = 0;
 	GList *work;
 
-	if (!func) return 0;
+	if (!ui->skin) return 0;
 
 	work = ui->skin->widget_list;
-	while(work)
+	while (work)
 		{
 		WidgetData *wd = work->data;
 
@@ -200,10 +201,63 @@ gint skin_widget_for_each_key(UIData *ui, const gchar *key, WidgetType type,
 		work = work->next;
 		}
 
-	/* editor updates */
-	if (ui->edit)
+	return count;
+}
+
+gint skin_widget_for_each_key(UIData *ui, const gchar *key, WidgetType type,
+			      void (*func)(WidgetData *wd, gpointer data, GdkPixbuf *pb, UIData *ui),
+			      gpointer data)
+{
+	gint count;
+	GList *work;
+	UIData *parent;
+
+	if (!func || !key) return 0;
+
+	if (ui->parent)
 		{
-		skin_widget_for_each_key(ui->edit, key, type, func, data);
+		parent = ui->parent;
+		}
+	else
+		{
+		parent = ui;
+		}
+
+	count = skin_widget_for_each_do(ui, key, type, func, data);
+
+	/* obey private keys */
+	if (ui_registered_key_is_private(ui, key, type))
+		{
+		return count;
+		}
+
+	/*
+	 * Children keys override the parent's,
+	 * so if at least one hit, do not propogate up to parent.
+	 */
+	if (parent != ui && count > 0) return count;
+
+	count += skin_widget_for_each_do(parent, key, type, func, data);
+
+	work = parent->children;
+	while (work)
+		{
+		UIData *child;
+
+		child = work->data;
+
+		if (child != ui)
+			{
+			count += skin_widget_for_each_do(child, key, type, func, data);
+			}
+
+		work = work->next;
+		}
+
+	/* editor updates (only parent can have an editor) */
+	if (parent->edit)
+		{
+		skin_widget_for_each_key(parent->edit, key, type, func, data);
 		}
 
 	return count;
@@ -216,7 +270,7 @@ static gint skin_widget_for_each_key_simple(SkinData *skin, const gchar *key, Wi
 	gint count = 0;
 	GList *work;
 
-	if (!func) return 0;
+	if (!func || !key) return 0;
 
 	work = skin->widget_list;
 	while(work)
@@ -252,6 +306,23 @@ SkinData *skin_new(void)
 
 	skin->background_filename = NULL;
 	skin->background_mask_filename = NULL;
+	skin->focus_filename = NULL;
+
+	skin->focus_overlay = NULL;
+	skin->focus_stretch = FALSE;
+	skin->focus_has_border = FALSE;
+	skin->focus_border_left = 0;
+	skin->focus_border_right = 0;
+	skin->focus_border_top = 0;
+	skin->focus_border_bottom = 0;
+	skin->focus_anchor_right = FALSE;
+	skin->focus_anchor_bottom = FALSE;
+
+	skin->focus_box_filled = FALSE;
+	skin->focus_box_alpha = 128;
+	skin->focus_box_r = 255;
+	skin->focus_box_g = 0;
+	skin->focus_box_b = 0;
 
 	return skin;
 }
@@ -275,8 +346,11 @@ void skin_free(SkinData *skin)
 
 	if (skin->real_overlay) gdk_pixbuf_unref(skin->real_overlay);
 
+	if (skin->focus_overlay) gdk_pixbuf_unref(skin->focus_overlay);
+
 	g_free(skin->background_filename);
 	g_free(skin->background_mask_filename);
+	g_free(skin->focus_filename);
 
 	g_free(skin);
 }
@@ -296,16 +370,42 @@ UIData *skin_get_ui(SkinData *skin)
 	return skin->ui;
 }
 
+static void skin_sync_back_on_widgets(SkinData *skin, UIData *ui)
+{
+	GList *work;
+
+	work = skin->widget_list;
+	while(work)
+		{
+		WidgetData *wd = work->data;
+		work = work->next;
+		ui_widget_sync_back(ui, wd);
+		}
+}
+
 void skin_sync_back(SkinData *skin, UIData *ui)
 {
 	GdkPixbuf *pb;
-	GList *work;
 
 	if (!skin || !skin->pixbuf) return;
 
 	pb = skin->pixbuf;
 
-	if (skin->transparent || slik_transparency_force)
+	if (ui->back_func &&
+	    (gdk_pixbuf_get_has_alpha(skin->overlay) || slik_transparency_force) &&
+	    ui->back_func(ui, pb, ui->back_data))
+		{
+		if (slik_transparency_force)
+			{
+			pixbuf_copy_area_alpha(skin->overlay, 0, 0, pb, 0, 0, skin->width, skin->height,
+					       slik_transparency_force_a);
+			}
+		else
+			{
+			pixbuf_copy_area_alpha(skin->overlay, 0, 0, pb, 0, 0, skin->width, skin->height, 255);
+			}
+		}
+	else if (skin->transparent || slik_transparency_force)
 		{
 		gint x, y;
 
@@ -335,13 +435,17 @@ void skin_sync_back(SkinData *skin, UIData *ui)
 		pixbuf_copy_area(skin->overlay, 0, 0, pb, 0, 0, skin->width, skin->height, FALSE);
 		}
 
-	work = skin->widget_list;
-	while(work)
-		{
-		WidgetData *wd = work->data;
-		work = work->next;
-		ui_widget_sync_back(ui, wd);
-		}
+	skin_sync_back_on_widgets(skin, ui);
+}
+
+void skin_set_underlay(SkinData *skin, UIData *ui, GdkPixbuf *pb)
+{
+	if (!skin || !skin->pixbuf || !pb) return;
+
+	pixbuf_copy_area(pb, 0, 0, skin->pixbuf, 0, 0, skin->width, skin->height, FALSE);
+	pixbuf_copy_area_alpha(skin->overlay, 0, 0, skin->pixbuf, 0, 0, skin->width, skin->height, 255);
+
+	skin_sync_back_on_widgets(skin, ui);
 }
 
 void skin_sync_buffer(SkinData *skin, UIData *ui)
@@ -350,6 +454,8 @@ void skin_sync_buffer(SkinData *skin, UIData *ui)
 
 	skin_back_setup(skin, ui);
 
+	if (!GTK_WIDGET_REALIZED(ui->display)) gtk_widget_realize(ui->display);
+
 	if (skin->buffer) gdk_pixmap_unref(skin->buffer);
 	skin->buffer = gdk_pixmap_new(ui->display->window, skin->width, skin->height, -1);
 
@@ -369,16 +475,21 @@ void skin_debug_print_registered_keys(SkinData *skin)
 		return;
 		}
 
-	printf("SKIN registered keys (%3d):\n", g_list_length(skin->widget_list));
-	printf("===========================\n");
+	printf("SKIN widget keys (%3d):\n", g_list_length(skin->widget_list));
+	printf("-[key]----------------[in bounds]-[type]-----[text id]--[data]-\n");
 
 	work = skin->widget_list;
 	while(work)
 		{
-		WidgetData *wd = work->data;
+		WidgetData *wd;
+		const gchar *data;
+
+		wd = work->data;
 		work = work->next;
 
-		printf("%-30s %-10s %s\n", wd->key, ui_widget_type_to_text(wd->type), wd->text_id);
+		data = ui_widget_get_data(wd, "data");
+
+		printf("%-30s %1d   %-10s %-10s %s\n", wd->key, wd->in_bounds, ui_widget_type_to_text(wd->type), (wd->text_id) ? wd->text_id : "", (data) ? data : "");
 		}
 }
 
@@ -588,4 +699,3 @@ void skin_finalize(SkinData *skin)
 					skin_resize_setup_cb, skin);
 }
 
-
