@@ -55,13 +55,20 @@ void font_ref(FontData *font);
 void font_unref(FontData *font);
 
 WidgetData *font_register(SkinData *skin, FontData *font, const gchar *key, const gchar *text_id);
+WidgetData *font_register_to_skin(const gchar *key, gchar **data, 
+				  gint extended,
+				  SkinData *skin, const gchar *text_id);
+
 WidgetType font_type_id(void);
 void font_type_init(void);
 
 
-TextData *text_new(FontData *font, gint length, gint sizeable, gint x, gint y);
-TextData *text_new_from_data(gchar **data, gint length, gint extended, gint sizeable, gint x, gint y);
-TextData *text_new_from_file(const gchar *file, gint length, gint extended, gint sizeable, gint x, gint y);
+TextData *text_new(FontData *font, gint x, gint y,
+		  gint length, gint sizeable);
+TextData *text_new_from_data(gchar **data, gint x, gint y,
+			     gint length, gint sizeable, gint extended);
+TextData *text_new_from_file(const gchar *file, gint x, gint y,
+			     gint length, gint sizeable, gint extended);
 void text_free(TextData *text);
 
 gint text_set_text(const gchar *key, UIData *ui, const gchar *new_text);
@@ -71,6 +78,10 @@ gint text_set_autoscroll(const gchar *key, UIData *ui, gint enable);
 gint text_set_scroll_speed(const gchar *key, UIData *ui, gint delay);
 
 WidgetData *text_register(SkinData *skin, TextData *text, const gchar *key, const gchar *text_id);
+WidgetData *text_register_to_skin(const gchar *key, gchar **data, gint x, gint y,
+				  gint length, gint sizeable, gint extended,
+				  SkinData *skin, const gchar *text_id);
+
 RegisterData *text_register_key(const gchar *key, UIData *ui,
 				const gchar *(*status_get_func)(TextData *text, const gchar *key, gpointer data), gpointer status_get_data);
 
