@@ -19,7 +19,7 @@
  *-------------------------------------
  */
 
-#define SLIK_VERSION "0.6.3"
+#define SLIK_VERSION "0.9.0"
 
 #define SKIN_SIZE_MIN 4
 #define SKIN_SIZE_MAX 2000
@@ -62,6 +62,11 @@ struct _WidgetObjectData
 	void (*func_set_coord)(gpointer widget, gint x, gint y);
 	void (*func_set_size)(gpointer widget, gint dev_w, gint dev_h);
 
+	/* focus stuff */
+	gint (*func_focus_draw)(gpointer widget, const gchar *key,
+				gint x, gint y, gint w, gint h, GdkPixbuf *pb, UIData *ui);
+	gint (*func_focus_key_event)(gpointer widget, const gchar *key, GdkEventKey *event, GdkPixbuf *pb, UIData *ui);
+
 	/* loading */
 	WidgetData *(*func_parse)(SkinData *skin, GList *list, const gchar *skin_dir, const gchar *key, gint edit);
 
@@ -149,12 +154,30 @@ struct _SkinData
 	gint height_max;
 	gint height_inc;
 
+	/* focus */
+	GdkPixbuf *focus_overlay;
+	gint focus_stretch;
+	gint focus_has_border;
+	gint focus_border_left;
+	gint focus_border_right;
+	gint focus_border_top;
+	gint focus_border_bottom;
+	gint focus_anchor_right;
+	gint focus_anchor_bottom;
+
+	gint focus_box_filled;
+	guint8 focus_box_alpha;
+	guint8 focus_box_r;
+	guint8 focus_box_g;
+	guint8 focus_box_b;
+
 	GList *widget_list;	/* widget list of WidgetData structs */
 
 	UIData *ui;		/* ui attached to */
 
 	gchar *background_filename;		/* used by editor */
 	gchar *background_mask_filename;	/* " */
+	gchar *focus_filename;
 };
 
 
@@ -164,6 +187,7 @@ struct _UIData
 	GtkWidget *display;
 
 	gchar *key;		/* key to reference ui */
+	gchar *class;		/* window class */
 
 	GList *register_list;	/* list of registered functions */
 	SkinData *skin;
@@ -177,6 +201,18 @@ struct _UIData
 	void (*click_func)(UIData *ui, gint button, guint32 time, gpointer data);
 	gpointer click_data;
 
+	/* default skin handler (used as last resort backup) */
+	SkinData *(*skin_func)(UIData *ui, const gchar *key, gpointer data);
+	gpointer skin_data;
+
+	/* called to request a background update, return TRUE if you set it */
+	gint (*back_func)(UIData *ui, GdkPixbuf *pixbuf, gpointer data);
+	gpointer back_data;
+
+	/* called when a new child window has opened */
+	void (*new_window_func)(UIData *ui, const gchar *key, gpointer data);
+	gpointer new_window_data;
+
 	WidgetData *active_widget;	/* widget with the mouse grab */
 	gint in_press;
 	gint in_move;
@@ -186,10 +222,16 @@ struct _UIData
 	gint press_root_x;
 	gint press_root_y;
 
+	WidgetData *focus_widget;
+
 	gint frozen;
 
 	gint root_win_idle;
 
+	/* groups */
+	UIData *parent;
+	GList *children;
+
 	UIData *edit;
 
 	gint destroyed;
@@ -250,13 +292,12 @@ struct _EditData
 	GList *widget_id_list;		/* list of 'widget type' data pointers */
 
 	GList *main_list;		/* list of 'add' backgrounds */
+	GList *focus_list;
 	gpointer main_page;		/* pointer to background page 'add' data */
 
 	/* help dialog */
 	GtkWidget *help_window;
-	GtkWidget *help_text;
 	gint help_follow;
-	gint help_lines;		/* crappy, broken set postion in GtkText */
 
 	gchar *help_path;
 };
@@ -271,8 +312,14 @@ struct _EditData
 extern gint slik_smart_placement;
 extern gint slik_remember_position;
 
+extern gint slik_focus_enable;
+
 extern gint slik_double_size;
 
+extern gint slik_scale;			/* set this before calling skin_parse to auto-scale a skin */
+extern gfloat slik_scale_value;		/* be carefull with value, some skins may crash when scaled */
+extern gint slik_allow_shapes;		/* set FALSE to disallow setting of shaped windows */
+
 extern gint slik_colorshift_r;
 extern gint slik_colorshift_g;
 extern gint slik_colorshift_b;
