@@ -148,7 +148,7 @@ void ui_widget_hide(UIData *ui, WidgetData *wd)
 	wd->hidden = TRUE;
 	if (ui_widget_get_geometry(wd, &x, &y, &w, &h))
 		{
-		ui_display_redraw_area(ui, x, y, w, h);
+		ui_display_redraw_area(ui, x, y, w, h, FALSE);
 		}
 }
 
@@ -160,23 +160,54 @@ void ui_widget_show(UIData *ui, WidgetData *wd)
 	ui_widget_draw(ui, wd, FALSE, TRUE);
 }
 
+/*
+ *-------------------------------
+ * widget focus funcs
+ *-------------------------------
+ */
+
+gint ui_widget_focus_draw(UIData *ui, WidgetData *wd, gint x, gint y, gint w, gint h)
+{
+	if (wd->od && wd->od->func_focus_draw)
+		{
+		return wd->od->func_focus_draw(wd->widget, wd->key, x, y, w, h, ui->skin->pixbuf, ui);
+		}
+
+	return FALSE;
+}
+
+gint ui_widget_focus_key_event(UIData *ui, WidgetData *wd, GdkEventKey *event)
+{
+	if (wd->od && wd->od->func_focus_key_event)
+		{
+		return wd->od->func_focus_key_event(wd->widget, wd->key, event, ui->skin->pixbuf, ui);
+		}
+
+	return FALSE;
+}
+
+gint ui_widget_can_focus(WidgetData *wd)
+{
+	return (wd->od && !wd->hidden && wd->in_bounds && wd->od->func_focus_key_event);
+}
+
 /*
  *-------------------------------
  * widget misc funcs
  *-------------------------------
  */
 
-gint ui_widget_for_each_key(UIData *ui, const gchar *key, WidgetType type,
-			    void (*func)(WidgetData *wd, gpointer data, UIData *ui),
-			    gpointer data)
+gint ui_widget_for_each_key_one(UIData *ui, const gchar *key, WidgetType type,
+			        void (*func)(WidgetData *wd, gpointer data, UIData *ui),
+			        gpointer data)
 {
 	gint count = 0;
 	GList *work;
 
-	if (!func) return 0;
+	if (!func || !ui->skin) return 0;
 
 	work = ui->skin->widget_list;
-	while(work)
+	while (work)
 		{
 		WidgetData *wd = work->data;
 
@@ -189,10 +220,43 @@ gint ui_widget_for_each_key(UIData *ui, const gchar *key, WidgetType type,
 		work = work->next;
 		}
 
-	/* editor updates */
-	if (ui->edit)
+	return count;
+}
+
+gint ui_widget_for_each_key(UIData *ui, const gchar *key, WidgetType type,
+			    void (*func)(WidgetData *wd, gpointer data, UIData *ui),
+			    gpointer data)
+{
+	gint count;
+	UIData *parent;
+	GList *work;
+
+	if (ui->parent)
 		{
-		ui_widget_for_each_key(ui->edit, key, type, func, data);
+		parent = ui->parent;
+		}
+	else
+		{
+		parent = ui;
+		}
+
+	count = ui_widget_for_each_key_one(parent, key, type, func, data);
+
+	work = parent->children;
+	while (work)
+		{
+		UIData *child;
+
+		child = work->data;
+		work = work->next;
+
+		count += ui_widget_for_each_key_one(child, key, type, func, data);
+		}
+
+	/* editor updates */
+	if (parent->edit)
+		{
+		ui_widget_for_each_key_one(parent->edit, key, type, func, data);
 		}
 
 	return count;
@@ -266,7 +330,7 @@ void ui_widget_set_coord(UIData *ui, WidgetData *wd, gint x, gint y, gint redraw
 	ui_widget_sync_back(ui, wd);
 
 	/* redraw old location */
-	if (old_bounds) ui_display_redraw_area(ui, ox, oy, ow, oh);
+	if (old_bounds) ui_display_redraw_area(ui, ox, oy, ow, oh, FALSE);
 
 	/* draw it */
 	ui_widget_draw(ui, wd, FALSE, TRUE);
@@ -297,7 +361,7 @@ void ui_widget_set_size(UIData *ui, WidgetData *wd, gint dev_w, gint dev_h, gint
 	ui_widget_sync_back(ui, wd);
 
 	/* redraw old location */
-	if (old_bounds) ui_display_redraw_area(ui, ox, oy, ow, oh);
+	if (old_bounds) ui_display_redraw_area(ui, ox, oy, ow, oh, FALSE);
 
 	/* draw it */
 	ui_widget_draw(ui, wd, FALSE, TRUE);
@@ -526,5 +590,3 @@ WidgetObjectData *ui_widget_object_copy(WidgetObjectData *src)
 	return od;
 }
 
-
-
