@@ -31,7 +31,8 @@
 
 static void menu_item_finish(GtkWidget *menu, GtkWidget *item, GtkSignalFunc func, gpointer data)
 {
-	if (func) gtk_signal_connect(GTK_OBJECT(item), "activate",(GtkSignalFunc) func, data);
+	if (func) gtk_signal_connect(GTK_OBJECT(item), "activate",
+				     GTK_SIGNAL_FUNC(func), data);
 	gtk_menu_append(GTK_MENU(menu), item);
 	gtk_widget_show(item);
 }
@@ -87,7 +88,7 @@ void menu_item_add_divider(GtkWidget *menu)
 static void popup_menu_short_lived_cb(GtkWidget *widget, gpointer data)
 {
 	/* destroy the menu */
-	gtk_widget_unref(GTK_WIDGET(data));
+	gtk_widget_destroy(GTK_WIDGET(data));
 }
 
 GtkWidget *popup_menu_short_lived(void)
@@ -95,7 +96,8 @@ GtkWidget *popup_menu_short_lived(void)
 	GtkWidget *menu;
 
 	menu = gtk_menu_new();
-	gtk_signal_connect(GTK_OBJECT(menu), "selection_done", popup_menu_short_lived_cb, menu);
+	gtk_signal_connect(GTK_OBJECT(menu), "selection_done",
+			   GTK_SIGNAL_FUNC(popup_menu_short_lived_cb), menu);
 	return menu;
 }
 
