@@ -200,12 +200,10 @@ static void dest_populate(Dest_Data *dd, const gchar *path)
 			if (p - 1 != filepath) p--;
 			p[0] = '\0';
 			}
-		else if (strcmp(path, "/") == 0)
-			{
-			filepath = g_strconcat("/", list->data, NULL);
-			}
 		else
-			filepath = g_strconcat(path, "/", list->data, NULL);
+			{
+			filepath = concat_dir_and_file(path, list->data);
+			}
 		
 		buf[0] = list->data;
 		row = gtk_clist_append(GTK_CLIST(dd->d_clist),buf);
@@ -229,7 +227,9 @@ static void dest_populate(Dest_Data *dd, const gchar *path)
         	        {
 			gint row;
 			gchar *filepath;
-			filepath = g_strconcat(path, "/", list->data, NULL);
+			const gchar *name = list->data;
+
+			filepath = concat_dir_and_file(path, name);
 		
 			buf[0] = list->data;
 			row = gtk_clist_append(GTK_CLIST(dd->f_clist),buf);
@@ -299,7 +299,7 @@ static gint dest_row_edit_rename_cb(ClistEditData *ced, const gchar *old, const
 	if (!old_path) return FALSE;
 
 	buf = remove_level_from_path(old_path);
-	new_path = g_strconcat(buf, "/", new, NULL);
+	new_path = concat_dir_and_file(buf, new);
 	g_free(buf);
 
 	if (isname(new_path))
@@ -434,10 +434,11 @@ static gint dest_press_cb(GtkWidget *widget, GdkEventButton *event, gpointer dat
 			clist_edit_set_highlight(dd->d_clist, row, TRUE);
 
 			menu = popup_menu_short_lived();
-			menu_item_add(menu, _("Rename"), dest_popup_dir_rename_cb, dd);
+			menu_item_add(menu, _("Rename"),
+				      GTK_SIGNAL_FUNC(dest_popup_dir_rename_cb), dd);
 
 			gtk_signal_connect(GTK_OBJECT(menu), "destroy",
-				GTK_SIGNAL_FUNC(dest_popup_dir_done_cb), dd);
+					   GTK_SIGNAL_FUNC(dest_popup_dir_done_cb), dd);
 			gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, event->time);
 			}
 		
@@ -457,11 +458,13 @@ static gint dest_press_cb(GtkWidget *widget, GdkEventButton *event, gpointer dat
 			clist_edit_set_highlight(dd->f_clist, row, TRUE);
 
 			menu = popup_menu_short_lived();
-			menu_item_add(menu, _("Rename"), dest_popup_file_rename_cb, dd);
-			menu_item_add(menu, _("Delete"), dest_popup_file_delete_cb, dd);
+			menu_item_add(menu, _("Rename"),
+				      GTK_SIGNAL_FUNC(dest_popup_file_rename_cb), dd);
+			menu_item_add(menu, _("Delete"),
+				      GTK_SIGNAL_FUNC(dest_popup_file_delete_cb), dd);
 
 			gtk_signal_connect(GTK_OBJECT(menu), "destroy",
-				GTK_SIGNAL_FUNC(dest_popup_file_done_cb), dd);
+					   GTK_SIGNAL_FUNC(dest_popup_file_done_cb), dd);
 			gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, event->time);
 
 			return FALSE;
@@ -504,7 +507,7 @@ static void dest_new_dir_cb(GtkWidget *widget, gpointer data)
 		}
 	else
 		{
-		buf = g_strconcat(dd->path, "/", _("New Directory"), NULL);
+		buf = concat_dir_and_file(dd->path, _("New Directory"));
 		path = unique_filename(buf, NULL, " ", FALSE);
 		g_free(buf);
 		}
@@ -752,17 +755,20 @@ GtkWidget *path_selection_new_with_files(GtkWidget *entry, const gchar *path,
 	gtk_widget_show(hbox2);
 
 	button = gtk_button_new_with_label(_("Home"));
-	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(dest_home_cb), dd);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(dest_home_cb), dd);
 	gtk_box_pack_start(GTK_BOX(hbox2), button, FALSE, FALSE, 0);
 	gtk_widget_show(button);
 
 	button = gtk_button_new_with_label(_("New Directory"));
-	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(dest_new_dir_cb), dd);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(dest_new_dir_cb), dd);
 	gtk_box_pack_start(GTK_BOX(hbox2), button, FALSE, FALSE, 0);
 	gtk_widget_show(button);
 
 	dd->hidden_button = gtk_check_button_new_with_label(_("Show hidden"));
-	gtk_signal_connect(GTK_OBJECT(dd->hidden_button), "clicked", GTK_SIGNAL_FUNC(dest_show_hidden_cb), dd);
+	gtk_signal_connect(GTK_OBJECT(dd->hidden_button), "clicked",
+			   GTK_SIGNAL_FUNC(dest_show_hidden_cb), dd);
 	gtk_box_pack_end(GTK_BOX(hbox2), dd->hidden_button, FALSE, FALSE, 0);
 	gtk_widget_show(dd->hidden_button);
 
@@ -780,9 +786,12 @@ GtkWidget *path_selection_new_with_files(GtkWidget *entry, const gchar *path,
 	dd->d_clist=gtk_clist_new(1);
 #endif
 	gtk_clist_set_column_auto_resize(GTK_CLIST(dd->d_clist), 0, TRUE);
-	gtk_signal_connect(GTK_OBJECT(dd->d_clist), "button_press_event", (GtkSignalFunc)dest_press_cb, dd);
-        gtk_signal_connect(GTK_OBJECT(dd->d_clist), "select_row", (GtkSignalFunc)dest_select_cb, dd);
-	gtk_signal_connect(GTK_OBJECT(dd->d_clist), "destroy", (GtkSignalFunc)dest_free_data, dd);
+	gtk_signal_connect(GTK_OBJECT(dd->d_clist), "button_press_event",
+			   GTK_SIGNAL_FUNC(dest_press_cb), dd);
+        gtk_signal_connect(GTK_OBJECT(dd->d_clist), "select_row",
+			   GTK_SIGNAL_FUNC(dest_select_cb), dd);
+	gtk_signal_connect(GTK_OBJECT(dd->d_clist), "destroy",
+			   GTK_SIGNAL_FUNC(dest_free_data), dd);
 	gtk_widget_set_usize(dd->d_clist, DEST_WIDTH, DEST_HEIGHT);
 	gtk_container_add (GTK_CONTAINER (scrolled), dd->d_clist);
         gtk_widget_show (dd->d_clist);
@@ -819,8 +828,10 @@ GtkWidget *path_selection_new_with_files(GtkWidget *entry, const gchar *path,
 #endif
 		gtk_clist_set_column_auto_resize(GTK_CLIST(dd->f_clist), 0, TRUE);
 		gtk_widget_set_usize(dd->f_clist, DEST_WIDTH, DEST_HEIGHT);
-		gtk_signal_connect(GTK_OBJECT(dd->f_clist), "button_press_event", (GtkSignalFunc)dest_press_cb, dd);
-		gtk_signal_connect(GTK_OBJECT(dd->f_clist), "select_row",(GtkSignalFunc) dest_select_cb, dd);
+		gtk_signal_connect(GTK_OBJECT(dd->f_clist), "button_press_event",
+				   GTK_SIGNAL_FUNC(dest_press_cb), dd);
+		gtk_signal_connect(GTK_OBJECT(dd->f_clist), "select_row",
+				   GTK_SIGNAL_FUNC(dest_select_cb), dd);
 		gtk_container_add(GTK_CONTAINER (scrolled), dd->f_clist);
 	        gtk_widget_show(dd->f_clist);
 
@@ -850,8 +861,13 @@ GtkWidget *path_selection_new_with_files(GtkWidget *entry, const gchar *path,
 		g_free(buf);
 		}
 
-	if (dd->filter_combo) gtk_signal_connect(GTK_OBJECT(GTK_COMBO(dd->filter_combo)->entry), "changed", dest_filter_changed_cb, dd);
-	gtk_signal_connect(GTK_OBJECT(dd->entry), "changed", dest_entry_changed_cb, dd);
+	if (dd->filter_combo)
+		{
+		gtk_signal_connect(GTK_OBJECT(GTK_COMBO(dd->filter_combo)->entry), "changed",
+				   GTK_SIGNAL_FUNC(dest_filter_changed_cb), dd);
+		}
+	gtk_signal_connect(GTK_OBJECT(dd->entry), "changed",
+			   GTK_SIGNAL_FUNC(dest_entry_changed_cb), dd);
 
 	return table;
 }
