@@ -449,11 +449,20 @@ static void tab_completion_popup_cb(GtkWidget *widget, gpointer data)
 	ptr = td->dir_path + strlen(td->dir_path) - 1;
 	buf = g_strconcat(td->dir_path, (ptr[0] == '/') ? "" : "/", name, NULL);
 	gtk_entry_set_text(GTK_ENTRY(td->entry), buf);
+	gtk_entry_set_position(GTK_ENTRY(td->entry), strlen(buf));
 	g_free(buf);
 
 	tab_completion_emit_tab_signal(td);
 }
 
+/*
+ * MAGIC_OFFSET: No simple way to determine frame size of menu.
+ * MAX: Limit popup menu items to a sane number, will be fixed when a list is used instead.
+ */
+
+#define TAB_COMP_POPUP_MAGIC_OFFSET 3
+#define TAB_COMP_POPUP_MAX 64
+
 static void tab_completion_popup_pos_cb(GtkMenu *menu, gint *x, gint *y, gpointer data)
 {
 	TabCompData *td = data;
@@ -465,7 +474,7 @@ static void tab_completion_popup_pos_cb(GtkMenu *menu, gint *x, gint *y, gpointe
 	*y += height;
 
 	*x += GTK_ENTRY(td->entry)->char_offset[GTK_EDITABLE(td->entry)->current_pos];
-	*x -= GTK_ENTRY(td->entry)->scroll_offset;
+	*x -= GTK_ENTRY(td->entry)->scroll_offset + TAB_COMP_POPUP_MAGIC_OFFSET;
 }
 
 static void tab_completion_popup_list(TabCompData *td, GList *list)
@@ -475,6 +484,7 @@ static void tab_completion_popup_list(TabCompData *td, GList *list)
 	GdkEvent *event;
 	guint32 etime;
 	gint ebutton;
+	gint count = 0;
 
 	if (!list) return;
 
@@ -488,7 +498,7 @@ static void tab_completion_popup_list(TabCompData *td, GList *list)
 	menu = popup_menu_short_lived();
 
 	work = list;
-	while (work)
+	while (work && count < TAB_COMP_POPUP_MAX)
 		{
 		gchar *name = work->data;
 		GtkWidget *item;
@@ -497,6 +507,7 @@ static void tab_completion_popup_list(TabCompData *td, GList *list)
 		gtk_object_set_data(GTK_OBJECT(item), "tab_completion_data", td);
 
 		work = work->next;
+		count++;
 		}
 
 	gtk_signal_connect(GTK_OBJECT(menu), "key_press_event",
@@ -536,7 +547,7 @@ static gint simple_sort(gconstpointer a, gconstpointer b)
 
 static gint tab_completion_do(TabCompData *td)
 {
-	gchar *entry_text = gtk_entry_get_text(GTK_ENTRY(td->entry));
+	const gchar *entry_text = gtk_entry_get_text(GTK_ENTRY(td->entry));
 	const gchar *entry_file;
 	gchar *entry_dir;
 	gchar *ptr;
@@ -773,7 +784,8 @@ static GtkWidget *tab_completion_create_complete_button(GtkWidget *entry)
 
 	button = gtk_button_new();
 	GTK_WIDGET_UNSET_FLAGS(button, GTK_CAN_FOCUS);
-	gtk_signal_connect(GTK_OBJECT(button), "clicked", (GtkSignalFunc) tab_completion_button_pressed, entry);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(tab_completion_button_pressed), entry);
 	
 	icon = gtk_pixmap_new(pixmap, mask);
 	gtk_container_add(GTK_CONTAINER(button), icon);
@@ -920,9 +932,9 @@ void tab_completion_add_to_entry(GtkWidget *entry, void (*enter_func)(const gcha
 	gtk_object_set_data(GTK_OBJECT(td->entry), "tab_completion_data", td);
 
 	gtk_signal_connect(GTK_OBJECT(entry), "key_press_event",
-			(GtkSignalFunc) tab_completion_key_pressed, td);
+			   GTK_SIGNAL_FUNC(tab_completion_key_pressed), td);
 	gtk_signal_connect(GTK_OBJECT(entry), "destroy",
-			(GtkSignalFunc) tab_completion_destroy, td);
+			   GTK_SIGNAL_FUNC(tab_completion_destroy), td);
 }
 
 void tab_completion_add_tab_func(GtkWidget *entry, void (*tab_func)(const gchar *, gpointer), gpointer data)
