@@ -104,7 +104,8 @@ static void generic_dialog_show_cb(GtkWidget *widget, gpointer data)
 	GenericDialog *gd = data;
 	if (gd->cancel_cb) generic_dialog_add(gd, _("Cancel"), gd->cancel_cb, FALSE);
 
-	gtk_signal_disconnect_by_func(GTK_OBJECT(widget), generic_dialog_show_cb, gd);
+	gtk_signal_disconnect_by_func(GTK_OBJECT(widget),
+				      GTK_SIGNAL_FUNC(generic_dialog_show_cb), gd);
 }
 
 void generic_dialog_add(GenericDialog *gd, const gchar *text,
@@ -114,7 +115,8 @@ void generic_dialog_add(GenericDialog *gd, const gchar *text,
 	button = gtk_button_new_with_label(text);
 	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
 	gtk_object_set_data(GTK_OBJECT(button), "dialog_function", func_cb);
-	gtk_signal_connect(GTK_OBJECT(button), "clicked", generic_dialog_click_cb, gd);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(generic_dialog_click_cb), gd);
 
 	gtk_container_add(GTK_CONTAINER(gd->hbox), button);
 
@@ -176,7 +178,8 @@ static void generic_dialog_setup(GenericDialog *gd,
 	gtk_box_pack_start(GTK_BOX(vbox), gd->hbox, FALSE, FALSE, 0);
 	gtk_widget_show(gd->hbox);
 
-	gtk_signal_connect(GTK_OBJECT(gd->dialog), "show", GTK_SIGNAL_FUNC(generic_dialog_show_cb), gd);
+	gtk_signal_connect(GTK_OBJECT(gd->dialog), "show",
+			   GTK_SIGNAL_FUNC(generic_dialog_show_cb), gd);
 }
 
 GenericDialog *generic_dialog_new(const gchar *title, const gchar *message,
@@ -295,7 +298,7 @@ void file_dialog_add_path_widgets(FileDialog *fd, const gchar *default_path, con
 
 		if (path)
 			{
-			fd->dest_path = g_strconcat(base, "/", path, NULL);
+			fd->dest_path = concat_dir_and_file(base, path);
 			}
 		else
 			{
@@ -319,7 +322,8 @@ void file_dialog_add_path_widgets(FileDialog *fd, const gchar *default_path, con
 		gtk_entry_set_position(GTK_ENTRY(fd->entry), strlen(fd->dest_path));
 		}
 
-	gtk_signal_connect(GTK_OBJECT(fd->entry), "changed", GTK_SIGNAL_FUNC(file_dialog_entry_cb), fd);
+	gtk_signal_connect(GTK_OBJECT(fd->entry), "changed",
+			   GTK_SIGNAL_FUNC(file_dialog_entry_cb), fd);
 }
 
 void file_dialog_add_filter(FileDialog *fd, const gchar *filter, const gchar *filter_desc, gint set)
